searchData={"items":[{"type":"module","title":"Kino","doc":"Client-driven interactive widgets for Livebook.\n\nKino is the library used by Livebook to render rich and interactive\noutputs directly from your Elixir code.","ref":"Kino.html"},{"type":"module","title":"Getting started - Kino","doc":"Livebook is distributed with a set of interactive tutorials and\nexamples, including some that specifically focus on Kino. If you're\njust getting started, going through these is highly recommended.\n\nYou can access these notebooks by starting Livebook and clicking\non \"Learn\" in the sidebar.","ref":"Kino.html#module-getting-started"},{"type":"module","title":"Built-in kinos - Kino","doc":"Kino provides several built-in kinos. The `Kino.Shorts` module\nprovides a facade to access and use most of the Kinos in this\nproject, although you can also use their modules directly as\nlisted in the sidebar.\n\nFor user interactions, `Kino.Input` and `Kino.Control` provide\na set of widgets for entering data and capturing user events.\nSee the respective module documentation for examples.\n\nKino also provides facilities to aid debugging, such as\n`Kino.Process` and a custom `dbg()` implementation that integrates\nwith Livebook.","ref":"Kino.html#module-built-in-kinos"},{"type":"module","title":"Custom kinos - Kino","doc":"Kino renders any data structure that implements the `Kino.Render`\nprotocol, falling back to the `Kernel.inspect/2` representation\nwhenever an implementation is not available. You can customize\nhow your own data structures are rendered by implementing the\n`Kino.Render` protocol.\n\nYou can also implement your own kinos by writing custom JavaScript,\nsee `Kino.JS` and `Kino.JS.Live` for more details.\n\n> #### Packaging {: .info}\n>\n> When publishing custom kinos and smart cells, please consider\n> the following guidelines:\n>\n>   * prefix package name with `kino_`, usually followed by the\n>     name of the integration, such as `kino_vega_lite`, `kino_ecto`\n>\n>   * namespace all modules under `KinoExample`, not `Kino.Example`.\n>     Note that official packages maintained by the Livebook team\n>     expose public APIs under `Kino.`, because they are essentially\n>     direct extensions of `Kino` and we make sure no conflicting\n>     modules exist. Unofficial packages should follow the usual\n>     Elixir conventions with respect to module names\n>","ref":"Kino.html#module-custom-kinos"},{"type":"function","title":"Kino.animate/2","doc":"Renders a kino that periodically calls the given function\nto render a new result.\n\nThe callback receives a stream element and should return a term\nto be rendered.\n\nThis function uses `Kino.Frame` as the underlying kino.\nIt returns nothing (a non-printable result).","ref":"Kino.html#animate/2"},{"type":"function","title":"Examples - Kino.animate/2","doc":"An animation is created by turning a stream of values into\nsubsequent animation frames:\n\n    Stream.interval(100)\n    |> Stream.take(100)\n    |> Kino.animate(fn i ->\n      Kino.Markdown.new(\"**Iteration: `#{i}`**\")\n    end)\n\nAlternatively an integer may be passed as a shorthand for\n`Stream.interval/1`:\n\n    # Render new Markdown every 100ms\n    Kino.animate(100, fn i ->\n      Kino.Markdown.new(\"**Iteration: `#{i}`**\")\n    end)","ref":"Kino.html#animate/2-examples"},{"type":"function","title":"Kino.animate/3","doc":"A stateful version of `animate/2`.\n\nThe callback receives a stream element and the accumulated state\nand it should return either of:\n\n  * `{:cont, term_to_render, state}` - the continue\n\n  * `:halt` - to no longer schedule callback evaluation","ref":"Kino.html#animate/3"},{"type":"function","title":"Examples - Kino.animate/3","doc":"This function is primarily useful to consume `Kino.Control` events:\n\n    button = Kino.Control.button(\"Click\")\n\n    button\n    |> Kino.Control.stream()\n    |> Kino.animate(0, fn _event, counter ->\n      new_counter = counter + 1\n      md = Kino.Markdown.new(\"**Clicks: `#{new_counter}`**\")\n      {:cont, md, new_counter}\n    end)","ref":"Kino.html#animate/3-examples"},{"type":"function","title":"Kino.async_listen/2","doc":"Same as `listen/2`, except each event is processed concurrently.","ref":"Kino.html#async_listen/2"},{"type":"function","title":"Kino.beam_paths/0","doc":"Returns the directories that contain `.beam` files for modules\ndefined in the notebook.","ref":"Kino.html#beam_paths/0"},{"type":"function","title":"Kino.configure/1","doc":"Configures Kino.\n\nThe supported options are:\n\n  * `:inspect`\n\nThey are discussed individually in the sections below.","ref":"Kino.html#configure/1"},{"type":"function","title":"Inspect - Kino.configure/1","doc":"A keyword list containing inspect options used for printing\nusual evaluation results. Defaults to pretty formatting with\na limit of 50 entries.\n\nTo show more entries, you configure a higher limit:\n\n    Kino.configure(inspect: [limit: 200])\n\nYou can also show all entries by setting the limit to `:infinity`,\nbut keep in mind that for large data structures it is memory-expensive\nand is not an advised configuration in this case. Instead prefer\nthe use of `IO.inspect/2` with `:infinity` limit when needed.\n\nSee `Inspect.Opts` for the full list of options.","ref":"Kino.html#configure/1-inspect"},{"type":"function","title":"Kino.inspect/2","doc":"Inspects the given term as cell output.\n\nThis works essentially the same as `IO.inspect/2`, except it\nalways produces colored text and respects the configuration\nset with `configure/1`.\n\nOpposite to `render/1`, it does not attempt to render the given\nterm as a kino.","ref":"Kino.html#inspect/2"},{"type":"function","title":"Kino.interrupt!/2","doc":"Interrupts evaluation with the given message.\n\nThis function raises a specific error to let Livebook known that\nevaluation should be stopped. The error message and a `Continue`\nbutton are shown to the user, who can then attempt to resolve the\nsource of the interrupt before resuming execution.\n\n> #### Do not use interrupt inside listeners {: .warning}\n>\n> Since `interrupt!/2` aborts the execution, it cannot be used\n> inside `Kino.listen/2` or other event listeners. In such cases,\n> you can use `Kino.Frame` and render any messages directly within\n> the frame, using `Kino.Text` or `Kino.Markdown`.","ref":"Kino.html#interrupt!/2"},{"type":"function","title":"Examples - Kino.interrupt!/2","doc":"text =\n      Kino.Input.text(\"Input\")\n      |> Kino.render()\n      |> Kino.Input.read()\n\n    if text == \"\" do\n      Kino.interrupt!(:error, \"Input required\")\n    end\n\n    # This will not be run if the `interrupt!` is called above\n    Kino.Markdown.new(\"**You entered:** #{text}\")","ref":"Kino.html#interrupt!/2-examples"},{"type":"function","title":"Kino.listen/2","doc":"Starts a process that consumes a stream with `fun` without blocking execution.\n\nIt returns the PID of the started process. The process can be terminated\nwith `Kino.terminate_child/1`.\n\nNote that events are processed by `fun` sequentially. If you want\nto process them concurrently, use `async_listen/2`.","ref":"Kino.html#listen/2"},{"type":"function","title":"Examples - Kino.listen/2","doc":"This function is primarily useful to consume `Kino.Control` events:\n\n    Kino.Control.button(\"Greet\")\n    |> Kino.listen(fn event -> IO.inspect(event) end)\n\nYou can also merge multiple controls into a single stream. For example,\nin order to merge them and tag each with a distinct event:\n\n    button = Kino.Control.button(\"Hello\")\n    input = Kino.Input.checkbox(\"Check\")\n\n    stream = Kino.Control.tagged_stream([hello: button, check: input])\n\n    Kino.listen(stream, fn\n      {:hello, event} -> ...\n      {:check, event} -> ...\n    end)\n\nAny other stream works as well:\n\n    Stream.interval(100)\n    |> Stream.take(10)\n    |> Kino.listen(fn i -> IO.puts(\"Ping #{i}\") end)\n\nFinally, an integer may be passed as a shorthand for `Stream.interval/1`:\n\n    Kino.listen(100, fn i -> IO.puts(\"Ping #{i}\") end)","ref":"Kino.html#listen/2-examples"},{"type":"function","title":"Kino.listen/3","doc":"A stateful version of `listen/2`.\n\nThe callback should return either of:\n\n  * `{:cont, state}` - the continue\n\n  * `:halt` - to stop listening","ref":"Kino.html#listen/3"},{"type":"function","title":"Examples - Kino.listen/3","doc":"button = Kino.Control.button(\"Click\")\n\n    Kino.listen(button, 0, fn _event, counter ->\n      new_counter = counter + 1\n      IO.puts(\"Clicks: #{new_counter}\")\n      {:cont, new_counter}\n    end)","ref":"Kino.html#listen/3-examples"},{"type":"function","title":"Kino.nothing/0","doc":"Returns a special value that results in no visible output.","ref":"Kino.html#nothing/0"},{"type":"function","title":"Examples - Kino.nothing/0","doc":"This is especially handy when you wish to suppress the default output\nof a cell. For instance, a cell containing this would normally result\nin verbose response output:\n\n    resp = Req.get!(\"https://example.org\")\n\nThat output can be suppressed by appending a call to `nothing/0`:\n\n    resp = Req.get!(\"https://example.org\")\n    Kino.nothing()","ref":"Kino.html#nothing/0-examples"},{"type":"function","title":"Kino.recompile/0","doc":"Recompiles dependenies.\n\nOnce you have installed dependencies with `Mix.install/1`, this will\nrecompile any outdated path dependencies declared during the install.\n\n> #### Reproducability {: .warning}\n>\n> Keep in mind that recompiling dependency modules is **not** going\n> to mark any cells as stale. This means that the given notebook\n> state may no longer be reproducable. This function is meant as a\n> utility when prototyping alongside a Mix project.","ref":"Kino.html#recompile/0"},{"type":"function","title":"Kino.render/1","doc":"Renders the given term as cell output.\n\nThis effectively allows any Livebook cell to have multiple\nevaluation results.","ref":"Kino.html#render/1"},{"type":"function","title":"Kino.start_child/1","doc":"Starts a process under the Kino supervisor.\n\nThe process is automatically terminated when the current process\nterminates or the current cell reevaluates.\n\nIf you want to terminate the started process, use\n`terminate_child/1`. If you terminate the process manually,\nthe Kino supervisor might restart it if the child's `:restart`\nstrategy says so.\n\n> #### Nested start {: .warning}\n>\n> It is not possible to use `start_child/1` while initializing\n> another process started this way. In other words, you generally\n> cannot call `start_child/1` inside callbacks such as `c:GenServer.init/1`\n> or `c:Kino.JS.Live.init/2`. If you do that, starting the process\n> will block forever.\n>\n> Note that creating many kinos uses `start_child/1` underneath,\n> hence the same restriction applies to starting those. See\n> `c:Kino.JS.Live.init/2` for more details.","ref":"Kino.html#start_child/1"},{"type":"function","title":"Kino.start_child!/1","doc":"Similar to `start_child/2` but returns the new pid or raises an error.","ref":"Kino.html#start_child!/1"},{"type":"function","title":"Kino.terminate_child/1","doc":"Terminates a child started with `start_child/1`.\n\nReturns `:ok` if the child was found and terminated, or\n`{:error, :not_found}` if the child was not found.","ref":"Kino.html#terminate_child/1"},{"type":"function","title":"Kino.tmp_dir/0","doc":"Returns a temporary directory that gets removed when the runtime\nterminates.","ref":"Kino.html#tmp_dir/0"},{"type":"type","title":"Kino.nothing/0","doc":"","ref":"Kino.html#t:nothing/0"},{"type":"module","title":"Kino.Control","doc":"Various widgets for user interactions.\n\nEach widget is a UI control element that the user interacts\nwith, consequently producing an event stream.\n\nThose widgets are often useful paired with `Kino.Frame` for\npresenting content that changes upon user interactions.","ref":"Kino.Control.html"},{"type":"module","title":"Examples - Kino.Control","doc":"First, create a control and make sure it is rendered,\neither by placing it at the end of a code cell or by\nexplicitly rendering it with `Kino.render/1`.\n\n    button = Kino.Control.button(\"Hello\")\n\nNext, events need to be received from the control. This can\nbe done either by subscribing a process to the control with\n`subscribe/2` or by creating an event stream using `stream/1`\nor `tagged_stream/1` and then registering a callback using\n`Kino.listen/2`.\n\nHere, we'll subscribe the current process to events:\n\n    Kino.Control.subscribe(button, :hello)\n\nAs the user clicks the button, the subscribed process\nreceives events:\n\n    IEx.Helpers.flush()\n    #=> {:hello, %{origin: \"client1\"}}\n    #=> {:hello, %{origin: \"client1\"}}","ref":"Kino.Control.html#module-examples"},{"type":"function","title":"Kino.Control.button/1","doc":"Creates a new button.","ref":"Kino.Control.html#button/1"},{"type":"function","title":"Examples - Kino.Control.button/1","doc":"Create the widget:\n\n    button = Kino.Control.button(\"Hello\")\n\nListen to events:\n\n    Kino.listen(button, fn event ->\n      ...\n    end)\n\nOr subscribe to them in a separate process:\n\n    Kino.Control.subscribe(button, :keyboard)","ref":"Kino.Control.html#button/1-examples"},{"type":"function","title":"Kino.Control.form/2","doc":"Creates a new form.\n\nA form is composed of regular inputs from the `Kino.Input` module,\nhowever in a form input values are not synchronized between users.\nConsequently, the form is another control for producing user-specific\nevents.\n\nEither `:submit` or `:report_changes` must be specified.","ref":"Kino.Control.html#form/2"},{"type":"function","title":"Options - Kino.Control.form/2","doc":"* `:submit` - specifies the label to use for the submit button\n    and enables submit events\n\n  * `:report_changes` - whether to send new form value whenever any\n    of the input changes. Defaults to `false`\n\n  * `:reset_on_submit` - a list of fields to revert to their default\n    values once the form is submitted. Use `true` to indicate all\n    fields. Defaults to `[]`","ref":"Kino.Control.html#form/2-options"},{"type":"function","title":"Event info - Kino.Control.form/2","doc":"In addition to standard properties, all events include additional\nproperties.\n\n  * `:type` - either `:submit` or `:change`\n\n  * `:data` - a map with field values, matching the field list","ref":"Kino.Control.html#form/2-event-info"},{"type":"function","title":"Examples - Kino.Control.form/2","doc":"Create a form out of inputs:\n\n    form =\n      Kino.Control.form(\n        [\n          name: Kino.Input.text(\"Name\"),\n          message: Kino.Input.textarea(\"Message\")\n        ],\n        submit: \"Send\"\n      )\n\nListen to events:\n\n    Kino.listen(form, fn event ->\n      ...\n    end)\n\nOr subscribe to them in a separate process:\n\n    Kino.Control.subscribe(form, :chat_form)\n\nAs users submit the form the payload is sent:\n\n    IEx.Helpers.flush()\n    #=> {:chat_form,\n    #=>   %{\n    #=>     data: %{message: \"Hola\", name: \"Amy\"},\n    #=>     origin: \"client1\",\n    #=>     type: :submit\n    #=>   }}\n    #=> {:chat_form,\n    #=>   %{\n    #=>     data: %{message: \"Hey!\", name: \"Jake\"},\n    #=>     origin: \"client2\",\n    #=>     type: :submit\n    #=>   }}","ref":"Kino.Control.html#form/2-examples"},{"type":"function","title":"Kino.Control.interval/1","doc":"Returns a new interval event source.\n\nThis can be used as event source for `stream/1` and `tagged_stream/1`.\nThe events are emitted periodically with an increasing value, starting\nfrom 0 and have the form:\n\n    %{type: :interval, iteration: non_neg_integer()}","ref":"Kino.Control.html#interval/1"},{"type":"function","title":"Kino.Control.keyboard/2","doc":"Creates a new keyboard control.\n\nThis widget is represented as button that toggles interception\nmode, in which the given keyboard events are captured.\n\n> #### Keyboard shortcut {:.info}\n>\n> As of Livebook v0.11, keyboard controls can be toggled by\n> focusing the cell and pressing `ctrl + k` (or `⌘ + k` on\n> MacOS).","ref":"Kino.Control.html#keyboard/2"},{"type":"function","title":"Options - Kino.Control.keyboard/2","doc":"Note that these options require Livebook v0.11 or later.\n\n  * `:default_handlers` - controls Livebook's default keyboard\n    shortcut handlers while the keyboard control is enabled.\n    Must be one of:\n\n    * `:off` (default) - all Livebook keyboard shortcuts are disabled\n\n    * `:on` - all Livebook keyboard shortcuts are enabled\n\n    * `:disable_only` - Livebook keyboard shortcuts are off except\n      for the shortcut to toggle (disable) the control","ref":"Kino.Control.html#keyboard/2-options"},{"type":"function","title":"Event info - Kino.Control.keyboard/2","doc":"In addition to standard properties, all events include additional\nproperties.","ref":"Kino.Control.html#keyboard/2-event-info"},{"type":"function","title":"Key events - Kino.Control.keyboard/2","doc":"* `:type` - either `:keyup` or `:keydown`\n\n  * `:key` - the value matching the browser [KeyboardEvent.key](https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/key)","ref":"Kino.Control.html#keyboard/2-key-events"},{"type":"function","title":"Status event - Kino.Control.keyboard/2","doc":"* `:type` - either `:status`\n\n  * `:enabled` - whether the keyboard is activated","ref":"Kino.Control.html#keyboard/2-status-event"},{"type":"function","title":"Examples - Kino.Control.keyboard/2","doc":"Create the widget:\n\n    keyboard = Kino.Control.keyboard([:keyup, :keydown, :status])\n\nListen to events:\n\n    Kino.listen(keyboard, fn event ->\n      ...\n    end)\n\nOr subscribe to them in a separate process:\n\n    Kino.Control.subscribe(keyboard, :keyboard)\n\nAs the user types events are streamed:\n\n    IEx.Helpers.flush()\n    #=> {:keyboard, %{enabled: true, origin: \"client1\", type: :status}\n    #=> {:keyboard, %{key: \"o\", origin: \"client1\", type: :keydown}}\n    #=> {:keyboard, %{key: \"k\", origin: \"client1\", type: :keydown}}\n    #=> {:keyboard, %{key: \"o\", origin: \"client1\", type: :keyup}}\n    #=> {:keyboard, %{key: \"k\", origin: \"client1\", type: :keyup}}","ref":"Kino.Control.html#keyboard/2-examples"},{"type":"function","title":"Kino.Control.stream/1","doc":"Merges several inputs and controls into a single `stream` of events.\n\nIt accepts a single source or a list of sources, where each\nsource is either of:\n\n  * `%Kino.Control{}` - emitting value on relevant interaction\n\n  * `%Kino.Input{}` - emitting value on value change\n\n  * `%Kino.JS.Live{}` - emitting value programmatically\n\n  * `t:interval/0` - emitting value periodically, see `interval/1`\n\nYou can then consume the stream to access its events.\nThe stream is typically consumed via `Kino.listen/2`.","ref":"Kino.Control.html#stream/1"},{"type":"function","title":"Example - Kino.Control.stream/1","doc":"button = Kino.Control.button(\"Hello\")\n    input = Kino.Input.checkbox(\"Check\")\n    interval = Kino.Control.interval(1000)\n\n    [button, input, interval]\n    |> Kino.Control.stream()\n    |> Kino.listen(fn event ->\n      IO.inspect(event)\n    end)\n    #=> %{type: :interval, iteration: 0}\n    #=> %{origin: \"client1\", type: :click}\n    #=> %{origin: \"client1\", type: :change, value: true}","ref":"Kino.Control.html#stream/1-example"},{"type":"function","title":"Kino.Control.subscribe/2","doc":"Subscribes the calling process to control or input events.\n\nThis is an alternative API to `stream/1`, such that event\nmessages are consumed via process messages instead of streams.\n\nThe events are sent as `{tag, info}`, where info is a map with\nevent details. In particular, it always includes `:origin`, which\nis an opaque identifier of the client that triggered the event.","ref":"Kino.Control.html#subscribe/2"},{"type":"function","title":"Kino.Control.tagged_stream/1","doc":"Same as `stream/1`, but attaches custom tag to every stream item.","ref":"Kino.Control.html#tagged_stream/1"},{"type":"function","title":"Example - Kino.Control.tagged_stream/1","doc":"button = Kino.Control.button(\"Hello\")\n    input = Kino.Input.checkbox(\"Check\")\n\n    [hello: button, check: input]\n    |> Kino.Control.tagged_stream()\n    |> Kino.listen(fn event ->\n      IO.inspect(event)\n    end)\n    #=> {:hello, %{origin: \"client1\", type: :click}}\n    #=> {:check, %{origin: \"client1\", type: :change, value: true}}","ref":"Kino.Control.html#tagged_stream/1-example"},{"type":"function","title":"Kino.Control.unsubscribe/1","doc":"Unsubscribes the calling process from control or input events.","ref":"Kino.Control.html#unsubscribe/1"},{"type":"type","title":"Kino.Control.event_source/0","doc":"","ref":"Kino.Control.html#t:event_source/0"},{"type":"opaque","title":"Kino.Control.interval/0","doc":"","ref":"Kino.Control.html#t:interval/0"},{"type":"opaque","title":"Kino.Control.t/0","doc":"","ref":"Kino.Control.html#t:t/0"},{"type":"module","title":"Kino.FS","doc":"Provides access to notebook files.","ref":"Kino.FS.html"},{"type":"function","title":"Kino.FS.file_path/1","doc":"Accesses notebook file with the given name and returns a local path\nto read its contents from.\n\nThis invocation may take a while, in case the file is downloaded\nfrom a URL and is not in the cache.\n\n> #### File operations {: .info}\n>\n> You should treat the file as read-only. To avoid unnecessary\n> copies the path may potentially be pointing to the original file,\n> in which case any write operations would be persisted. This\n> behaviour is not always the case, so you should not rely on it\n> either.","ref":"Kino.FS.html#file_path/1"},{"type":"function","title":"Kino.FS.file_spec/1","doc":"Accesses notebook file with the given name and returns a specification\nof the file location.\n\nThis does not copy any files and moves the responsibility of reading\nthe file to the caller. If you need to read a file directly, use\n`file_path/1`.","ref":"Kino.FS.html#file_spec/1"},{"type":"module","title":"Kino.Frame","doc":"A placeholder for outputs.\n\nA frame wraps outputs that can be dynamically updated at\nany time.\n\nAlso see `Kino.animate/3` which offers a convenience on\ntop of this kino.","ref":"Kino.Frame.html"},{"type":"module","title":"Examples - Kino.Frame","doc":"frame = Kino.Frame.new() |> Kino.render()\n\n    for i <- 1..100 do\n      Kino.Frame.render(frame, i)\n      Process.sleep(50)\n    end\n\nOr with a scheduled task in the background.\n\n    frame = Kino.Frame.new() |> Kino.render()\n\n    Kino.listen(50, fn i ->\n      Kino.Frame.render(frame, i)\n    end)","ref":"Kino.Frame.html#module-examples"},{"type":"function","title":"Kino.Frame.append/3","doc":"Renders and appends the given term to the frame.","ref":"Kino.Frame.html#append/3"},{"type":"function","title":"Options - Kino.Frame.append/3","doc":"* `:to` - the client id to whom the update is directed. This\n    option is useful when updating frame in response to client\n    events, such as form submission\n\n  * `:temporary` - when `true`, the update is applied only to\n    the connected clients and doesn't become a part of frame\n    history. Defaults to `false`, unless `:to` is given. Direct\n    updates are never a part of frame history","ref":"Kino.Frame.html#append/3-options"},{"type":"function","title":"Kino.Frame.clear/2","doc":"Removes all outputs within the given frame.","ref":"Kino.Frame.html#clear/2"},{"type":"function","title":"Options - Kino.Frame.clear/2","doc":"* `:to` - the client id to whom the update is directed. This\n    option is useful when updating frame in response to client\n    events, such as form submission\n\n  * `:temporary` - when `true`, the update is applied only to\n    the connected clients and doesn't become a part of frame\n    history. Defaults to `false`, unless `:to` is given. Direct\n    updates are never a part of frame history","ref":"Kino.Frame.html#clear/2-options"},{"type":"function","title":"Kino.Frame.new/1","doc":"Creates a new frame.","ref":"Kino.Frame.html#new/1"},{"type":"function","title":"Options - Kino.Frame.new/1","doc":"* `:placeholder` - whether to render a placeholder when the frame\n    is empty. Defaults to `true`","ref":"Kino.Frame.html#new/1-options"},{"type":"function","title":"Kino.Frame.render/3","doc":"Renders the given term within the frame.\n\nThis works similarly to `Kino.render/1`, but the rendered\noutput replaces existing frame contents.","ref":"Kino.Frame.html#render/3"},{"type":"function","title":"Options - Kino.Frame.render/3","doc":"* `:to` - the client id to whom the update is directed. This\n    option is useful when updating frame in response to client\n    events, such as form submission\n\n  * `:temporary` - when `true`, the update is applied only to\n    the connected clients and doesn't become a part of frame\n    history. Defaults to `false`, unless `:to` is given. Direct\n    updates are never a part of frame history","ref":"Kino.Frame.html#render/3-options"},{"type":"opaque","title":"Kino.Frame.t/0","doc":"","ref":"Kino.Frame.html#t:t/0"},{"type":"module","title":"Kino.Input","doc":"Various input elements for entering data.","ref":"Kino.Input.html"},{"type":"module","title":"Examples - Kino.Input","doc":"First, create an input and make sure it is rendered,\neither by placing it at the end of a code cell or by\nexplicitly rendering it with `Kino.render/1`.\n\n    input = Kino.Input.text(\"Name\")\n\nThen read the value after the input has been rendered:\n\n    name = Kino.Input.read(input)\n\nAll inputs are shared by default: once you change the input,\nyour changes will be immediately replicated to all users\nreading the notebook. Use `Kino.Control.form/2` if you want\neach user to have their own input.","ref":"Kino.Input.html#module-examples"},{"type":"module","title":"Async API - Kino.Input","doc":"You can subscribe to input changes or use the `Stream`\nAPI for event feed. See the `Kino.Control` module for\nmore details.","ref":"Kino.Input.html#module-async-api"},{"type":"function","title":"Kino.Input.audio/2","doc":"Creates a new audio input.\n\nThe input value is a map, with an audio file and metadata:\n\n    %{\n      file_ref: term(),\n      num_channels: pos_integer(),\n      sampling_rate: pos_integer(),\n      format: :pcm_f32 | :wav\n    }\n\nNote that the value can also be `nil`, if no audio is selected.\n\nThe file path can then be accessed using `file_path/1`.\n\n> #### Warning {: .warning}\n>\n> The audio input is shared by default: once you upload an audio,\n> the audio will be replicated to all users reading the notebook.\n> Use `Kino.Control.form/2` if you want each user to have a distinct\n> audio upload with an explicit submission button.","ref":"Kino.Input.html#audio/2"},{"type":"function","title":"Options - Kino.Input.audio/2","doc":"* `:format` - the format to read the audio as, either of:\n\n    * `:pcm_f32` (default) - the PCM (32-bit float) format. Note that\n      the binary uses native system endianness. Such binary can be\n      directly converted to an `Nx` tensor, with no additional decoding\n\n    * `:wav`\n\n  * `:sampling_rate` - the sampling rate (samples per second) of\n    the audio data. Defaults to `48_000`","ref":"Kino.Input.html#audio/2-options"},{"type":"function","title":"Kino.Input.checkbox/2","doc":"Creates a new checkbox.\n\nThe input value can be either `true` or `false`.","ref":"Kino.Input.html#checkbox/2"},{"type":"function","title":"Options - Kino.Input.checkbox/2","doc":"* `:default` - the initial input value. Defaults to `false`","ref":"Kino.Input.html#checkbox/2-options"},{"type":"function","title":"Kino.Input.color/2","doc":"Creates a new color input.\n\nThe input value can be a hex color string.","ref":"Kino.Input.html#color/2"},{"type":"function","title":"Options - Kino.Input.color/2","doc":"* `:default` - the initial input value. Defaults to `#6583FF`\n\n  * `:debounce` - determines when input changes are emitted. When\n    set to `:blur`, the change propagates when the user leaves the\n    input. When set to a non-negative number of milliseconds, the\n    change propagates after the specified delay. Defaults to `:blur`","ref":"Kino.Input.html#color/2-options"},{"type":"function","title":"Kino.Input.date/2","doc":"Creates a new date input.\n\nThe input is read as a `%Date{}` struct.","ref":"Kino.Input.html#date/2"},{"type":"function","title":"Options - Kino.Input.date/2","doc":"* `:default` - the initial input value. Defaults to `nil`\n\n  * `:min` - the minimum date value\n\n  * `:max` - the maximum date value","ref":"Kino.Input.html#date/2-options"},{"type":"function","title":"Kino.Input.file/2","doc":"Creates a new file input.\n\nThe input value is a map, with a file and metadata:\n\n    %{\n      file_ref: term(),\n      client_name: String.t()\n    }\n\nNote that the value can also be `nil`, if no file is selected.\n\nThe file path can then be accessed using `file_path/1`.\n\n> #### Warning {: .warning}\n>\n> The file input is shared by default: once you upload a file,\n> the file will be replicated to all users reading the notebook.\n> Use `Kino.Control.form/2` if you want each user to have a distinct\n> file upload with an explicit submission button.","ref":"Kino.Input.html#file/2"},{"type":"function","title":"Considerations - Kino.Input.file/2","doc":"Note that a file may be deleted in certain cases, specifically:\n\n  * when the file is reuploaded\n  * when used with a form and the uploading user leaves\n  * when the input is removed\n\nThe deletion is not immediate and you are unlikely to run into this\nin practice, however theoretically `file_path/1` may point to a\nnon-existing file.","ref":"Kino.Input.html#file/2-considerations"},{"type":"function","title":"Options - Kino.Input.file/2","doc":"* `:accept` - the list of accepted file types (either extensions\n    or MIME types) or `:any`. Defaults to `:any`","ref":"Kino.Input.html#file/2-options"},{"type":"function","title":"Examples - Kino.Input.file/2","doc":"To read the content of currently uploaded file we would do:\n\n    # [Cell 1]\n\n    input = Kino.Input.file(\"File\")\n\n    # [Cell 2]\n\n    value = Kino.Input.read(input)\n    path = Kino.Input.file_path(value.file_ref)\n    File.read!(path)\n\nAnd here's how we could process an asynchronous form submission:\n\n    # [Cell 1]\n\n    form = Kino.Control.form([file: Kino.Input.file(\"File\")], submit: \"Send\")\n\n    # [Cell 2]\n\n    form\n    |> Kino.Control.stream()\n    |> Kino.listen(fn event ->\n      path = Kino.Input.file_path(event.data.file.file_ref)\n      content = File.read!(path)\n      IO.inspect(content)\n    end)","ref":"Kino.Input.html#file/2-examples"},{"type":"function","title":"Kino.Input.file_path/1","doc":"Returns file path for the given file identifier.","ref":"Kino.Input.html#file_path/1"},{"type":"function","title":"Kino.Input.image/2","doc":"Creates a new image input.\n\nThe input value is a map, with an image file and metadata:\n\n    %{\n      file_ref: term(),\n      height: pos_integer(),\n      width: pos_integer(),\n      format: :rgb | :png | :jpeg\n    }\n\nNote that the value can also be `nil`, if no image is selected.\n\nThe file path can then be accessed using `file_path/1`.\n\n> #### Warning {: .warning}\n>\n> The image input is shared by default: once you upload a image,\n> the image will be replicated to all users reading the notebook.\n> Use `Kino.Control.form/2` if you want each user to have a distinct\n> image upload with an explicit submission button.","ref":"Kino.Input.html#image/2"},{"type":"function","title":"Options - Kino.Input.image/2","doc":"* `:format` - the format to read the image as, either of:\n\n    * `:rgb` (default) - the binary includes raw pixel values, each\n      encoded as a single byte in the HWC order. Such binary can be\n      directly converted to an `Nx` tensor, with no additional decoding\n\n    * `:png`\n\n    * `:jpeg` (or `:jpg`)\n\n  * `:size` - the size to fit the image into, given as `{height, width}`\n\n  * `:fit` - the strategy of fitting the image into `:size`, either of:\n\n    * `:contain` (default) - resizes the image, such that it fits in\n      a box of `:size`, but preserving the aspect ratio. The resulting\n      image can be smaller or equal to `:size`\n\n    * `:match` - resizes the image to `:size`, with no respect for\n      aspect ratio\n\n    * `:pad` - same as `:contain`, but pads the image to match `:size`\n      exactly\n\n    * `:crop` - resizes the image, such that one edge fits in `:size`\n      and the other overflows, then center-crops the image to match\n      `:size` exactly","ref":"Kino.Input.html#image/2-options"},{"type":"function","title":"Kino.Input.number/2","doc":"Creates a new number input.\n\nThe input value is can be either a number or `nil`.","ref":"Kino.Input.html#number/2"},{"type":"function","title":"Options - Kino.Input.number/2","doc":"* `:default` - the initial input value. Defaults to `nil`\n\n  * `:debounce` - determines when input changes are emitted. When\n    set to `:blur`, the change propagates when the user leaves the\n    input. When set to a non-negative number of milliseconds, the\n    change propagates after the specified delay. Defaults to `:blur`","ref":"Kino.Input.html#number/2-options"},{"type":"function","title":"Kino.Input.password/2","doc":"Creates a new password input.\n\nThis is similar to text input, except the content is not\nvisible by default.","ref":"Kino.Input.html#password/2"},{"type":"function","title":"Options - Kino.Input.password/2","doc":"* `:default` - the initial input value. Defaults to `\"\"`\n\n  * `:debounce` - determines when input changes are emitted. When\n    set to `:blur`, the change propagates when the user leaves the\n    input. When set to a non-negative number of milliseconds, the\n    change propagates after the specified delay. Defaults to `:blur`","ref":"Kino.Input.html#password/2-options"},{"type":"function","title":"Kino.Input.range/2","doc":"Creates a new slider input.\n\nThe input value can be either float in the configured range.","ref":"Kino.Input.html#range/2"},{"type":"function","title":"Options - Kino.Input.range/2","doc":"* `:default` - the initial input value. Defaults to the\n    minimum value\n\n  * `:min` - the minimum value\n\n  * `:max` - the maximum value\n\n  * `:step` - the slider increment\n\n  * `:debounce` - determines when input changes are emitted. When\n    set to a non-negative number of milliseconds, the change propagates\n    after the specified delay. Defaults to `250`","ref":"Kino.Input.html#range/2-options"},{"type":"function","title":"Kino.Input.read/1","doc":"Synchronously reads the current input value.","ref":"Kino.Input.html#read/1"},{"type":"function","title":"Examples - Kino.Input.read/1","doc":"input =\n      Kino.Input.text(\"Name\")\n      |> Kino.render()\n\n    Kino.Input.read(input)","ref":"Kino.Input.html#read/1-examples"},{"type":"function","title":"Kino.Input.select/3","doc":"Creates a new select input.\n\nThe input expects a list of options in the form `[{value, label}]`,\nwhere `value` is an arbitrary term and `label` is a descriptive\nstring.","ref":"Kino.Input.html#select/3"},{"type":"function","title":"Options - Kino.Input.select/3","doc":"* `:default` - the initial input value. Defaults to the first\n    value from the given list of options","ref":"Kino.Input.html#select/3-options"},{"type":"function","title":"Examples - Kino.Input.select/3","doc":"Kino.Input.select(\"Language\", [en: \"English\", fr: \"Français\"])\n\n    Kino.Input.select(\"Language\", [{1, \"One\"}, {2, \"Two\"}, {3, \"Three\"}])","ref":"Kino.Input.html#select/3-examples"},{"type":"function","title":"Kino.Input.text/2","doc":"Creates a new text input.","ref":"Kino.Input.html#text/2"},{"type":"function","title":"Options - Kino.Input.text/2","doc":"* `:default` - the initial input value. Defaults to `\"\"`\n\n  * `:debounce` - determines when input changes are emitted. When\n    set to `:blur`, the change propagates when the user leaves the\n    input. When set to a non-negative number of milliseconds, the\n    change propagates after the specified delay. Defaults to `:blur`","ref":"Kino.Input.html#text/2-options"},{"type":"function","title":"Kino.Input.textarea/2","doc":"Creates a new multiline text input.","ref":"Kino.Input.html#textarea/2"},{"type":"function","title":"Options - Kino.Input.textarea/2","doc":"* `:default` - the initial input value. Defaults to `\"\"`\n\n  * `:monospace` - whether to use a monospace font inside the textarea.\n    Defaults to `false`\n\n  * `:debounce` - determines when input changes are emitted. When\n    set to `:blur`, the change propagates when the user leaves the\n    input. When set to a non-negative number of milliseconds, the\n    change propagates after the specified delay. Defaults to `:blur`","ref":"Kino.Input.html#textarea/2-options"},{"type":"function","title":"Kino.Input.url/2","doc":"Creates a new URL input.\n\nThe input value can be either a valid URL string or `nil`.","ref":"Kino.Input.html#url/2"},{"type":"function","title":"Options - Kino.Input.url/2","doc":"* `:default` - the initial input value. Defaults to `nil`\n\n  * `:debounce` - determines when input changes are emitted. When\n    set to `:blur`, the change propagates when the user leaves the\n    input. When set to a non-negative number of milliseconds, the\n    change propagates after the specified delay. Defaults to `:blur`","ref":"Kino.Input.html#url/2-options"},{"type":"function","title":"Kino.Input.utc_datetime/2","doc":"Creates a new datetime input.\n\nThe input is editable in user-local time zone, however the value\nis always read in UTC as a `%NaiveDateTime{}` struct.","ref":"Kino.Input.html#utc_datetime/2"},{"type":"function","title":"Options - Kino.Input.utc_datetime/2","doc":"* `:default` - the initial input value. Defaults to `nil`\n\n  * `:min` - the minimum datetime value (in UTC)\n\n  * `:max` - the maximum datetime value (in UTC)","ref":"Kino.Input.html#utc_datetime/2-options"},{"type":"function","title":"Kino.Input.utc_time/2","doc":"Creates a new time input.\n\nThe input is editable in user-local time zone, however the value\nis always read in UTC as a `%Time{}` struct.","ref":"Kino.Input.html#utc_time/2"},{"type":"function","title":"Options - Kino.Input.utc_time/2","doc":"* `:default` - the initial input value. Defaults to `nil`\n\n  * `:min` - the minimum time value (in UTC)\n\n  * `:max` - the maximum time value (in UTC)","ref":"Kino.Input.html#utc_time/2-options"},{"type":"opaque","title":"Kino.Input.t/0","doc":"","ref":"Kino.Input.html#t:t/0"},{"type":"module","title":"Kino.Process","doc":"This module contains kinos for generating visualizations to help\nintrospect your running processes.","ref":"Kino.Process.html"},{"type":"function","title":"Kino.Process.app_tree/2","doc":"Generates a visualization of an application tree.\n\nGiven the name of an application as an atom, this function will render the\napplication tree. It is displayed with solid lines denoting supervisor-worker\nrelationships and dotted lines denoting links between processes. The graph\nrendering supports the following options:","ref":"Kino.Process.html#app_tree/2"},{"type":"function","title":"Options - Kino.Process.app_tree/2","doc":"* `:direction` - defines the direction of the graph visual. The\n    value can either be `:top_down` or `:left_right`. Defaults to `:top_down`.\n\n  * `:render_ets_tables` - determines whether ETS tables associated with the\n    supervision tree are rendered. Defaults to `false`.","ref":"Kino.Process.html#app_tree/2-options"},{"type":"function","title":"Examples - Kino.Process.app_tree/2","doc":"To view the applications running in your instance run:\n\n    :application_controller.which_applications()\n\nYou can then call `Kino.Process.app_tree/1` to render\nthe application tree using using the atom of the application.\n\n    Kino.Process.app_tree(:logger)\n\nYou can also change the direction of the rendering by calling\n`Kino.Process.app_tree/2` with the `:direction` option.\n\n    Kino.Process.app_tree(:logger, direction: :left_right)","ref":"Kino.Process.html#app_tree/2-examples"},{"type":"function","title":"Kino.Process.render_app_tree/2","doc":"Renders a visual of the provided application tree.\n\nThis function renders an application tree much like `app_tree/2` with the difference\nbeing that this function can be called anywhere within the Livebook code block\nwhereas `app_tree/2` must have its result be the last thing returned from the code\nblock in order to render the visual. It supports the same options as `app_tree/2` as\nit delegates to that function to generate the visual.","ref":"Kino.Process.html#render_app_tree/2"},{"type":"function","title":"Kino.Process.render_seq_trace/3","doc":"Renders a sequence diagram of process messages and returns the function result.\n\nThis function renders a sequence diagram much like `seq_trace/2` with the difference\nbeing that this function can be called anywhere within the Livebook code block\nwhereas `seq_trace/2` must have its result be the last thing returned from the code\nblock in order to render the visual. In addition, this function returns the result\nfrom the provided trace function.","ref":"Kino.Process.html#render_seq_trace/3"},{"type":"function","title":"Kino.Process.render_sup_tree/2","doc":"Renders a visual of the provided supervision tree.\n\nThis function renders a supervision tree much like `sup_tree/2` with the difference\nbeing that this function can be called anywhere within the Livebook code block\nwhereas `sup_tree/2` must have its result be the last thing returned from the code\nblock in order to render the visual. It supports the same options as `sup_tree/2` as\nit delegates to that function to generate the visual.","ref":"Kino.Process.html#render_sup_tree/2"},{"type":"function","title":"Kino.Process.seq_trace/3","doc":"Generate a sequence diagram of process messages starting from `self()`.\n\nThe provided function is executed and traced, with all the events sent to and\nreceived by the trace target processes rendered in a sequence diagram. The trace\ntarget argument can either be a single PID, a list of PIDs, or the atom `:all`\ndepending on what messages you would like to retain in your trace.","ref":"Kino.Process.html#seq_trace/3"},{"type":"function","title":"Options - Kino.Process.seq_trace/3","doc":"* `:message_label` - A function to help label message events. If\n    the given function returns `:continue`, then the default label\n    is used. However, if the function returns a `String.t()`, then\n    that will be used for the label.","ref":"Kino.Process.html#seq_trace/3-options"},{"type":"function","title":"Examples - Kino.Process.seq_trace/3","doc":"To generate a trace of all the messages occurring during the execution of the\nprovided function, you can do the following:\n\n    Kino.Process.seq_trace(fn ->\n      {:ok, agent_pid} = Agent.start_link(fn -> [] end)\n      Process.monitor(agent_pid)\n\n      1..2\n      |> Task.async_stream(\n        fn value ->\n          Agent.get(agent_pid, fn value -> value end)\n          100 * value\n        end,\n        max_concurrency: 3\n      )\n      |> Stream.run()\n\n      Agent.stop(agent_pid)\n    end)\n\nIf you are only interested in messages being sent to or received by certain PIDs,\nyou can filter the sequence diagram by specifying the PIDs that you are interested\nin:\n\n    {:ok, agent_pid} = Agent.start_link(fn -> [] end)\n    Process.monitor(agent_pid)\n\n    Kino.Process.seq_trace(agent_pid, fn ->\n      1..2\n      |> Task.async_stream(\n        fn value ->\n          Agent.get(agent_pid, fn value -> value end)\n          100 * value\n        end,\n        max_concurrency: 3\n      )\n      |> Stream.run()\n\n      Agent.stop(agent_pid)\n    end)\n\nFurther if you are interested in custom labeling between messages\nsent between processes, you can specify custom labels for the\nmessages you are interested in:\n\n    {:ok, agent_pid} = Agent.start_link(fn -> [] end)\n    Process.monitor(agent_pid)\n\n    Kino.Process.seq_trace(agent_pid, fn ->\n      1..2\n      |> Task.async_stream(\n        fn value ->\n          Agent.get(agent_pid, fn value -> value end)\n          100 * value\n        end,\n        max_concurrency: 3\n      )\n      |> Stream.run()\n\n      Agent.stop(agent_pid)\n    end,\n    message_label: fn(msg) ->\n      case msg do\n        {:\"$gen_call\", _ref, {:get, _}} -> {:ok, \"GET: value\"}\n        _ -> :continue\n      end\n  end)","ref":"Kino.Process.html#seq_trace/3-examples"},{"type":"function","title":"Kino.Process.sup_tree/2","doc":"Generates a visualization of a supervision tree.\n\nThe provided supervisor can be either a named process or a PID. The supervision tree\nis displayed with solid lines denoting supervisor-worker relationships and dotted\nlines denoting links between processes. The graph rendering supports the following\noptions:","ref":"Kino.Process.html#sup_tree/2"},{"type":"function","title":"Options - Kino.Process.sup_tree/2","doc":"* `:direction` - defines the direction of the graph visual. The\n    value can either be `:top_down` or `:left_right`. Defaults to `:top_down`.","ref":"Kino.Process.html#sup_tree/2-options"},{"type":"function","title":"Examples - Kino.Process.sup_tree/2","doc":"With a supervisor definition like so:\n\n    {:ok, supervisor_pid} =\n      Supervisor.start_link(\n        [\n          {DynamicSupervisor, strategy: :one_for_one, name: MyApp.DynamicSupervisor},\n          {Agent, fn -> [] end}\n        ],\n        strategy: :one_for_one,\n        name: MyApp.Supervisor\n      )\n\n    Enum.each(1..3, fn _ ->\n      DynamicSupervisor.start_child(MyApp.DynamicSupervisor, {Agent, fn -> %{} end})\n    end)\n\nYou can then call `Kino.Process.sup_tree/1` to render\nthe supervision tree using using the PID of the supervisor.\n\n    Kino.Process.sup_tree(supervisor_pid)\n\nYou can also render the supervisor by passing the name of the supervisor\nif the supervisor was started with a name.\n\n    Kino.Process.sup_tree(MyApp.Supervisor)\n\nYou can also change the direction of the rendering by calling\n`Kino.Process.sup_tree/2` with the `:direction` option.\n\n    Kino.Process.sup_tree(MyApp.Supervisor, direction: :left_right)","ref":"Kino.Process.html#sup_tree/2-examples"},{"type":"type","title":"Kino.Process.label_response/0","doc":"","ref":"Kino.Process.html#t:label_response/0"},{"type":"type","title":"Kino.Process.supervisor/0","doc":"","ref":"Kino.Process.html#t:supervisor/0"},{"type":"type","title":"Kino.Process.trace_target/0","doc":"","ref":"Kino.Process.html#t:trace_target/0"},{"type":"module","title":"Kino.Proxy","doc":"Functionality for handling proxy requests forwarded from Livebook.\n\nLivebook proxies requests at the following paths:\n\n  * `/sessions/:id/proxy/*path` - a notebook session\n\n  * `/apps/:slug/:session_id/proxy/*path` - a specific app session\n\n  * `/apps/:slug/proxy/*path` - generic app path, only supported for\n    single-session apps. If the app has automatic shutdowns enabled\n    and it is not currently running, it will be automatically started\n\nYou can define a custom listener to handle requests at these paths.\nThe listener receives a `Plug.Conn` and it should use the `Plug` API\nto send the response, for example:\n\n    Kino.Proxy.listen(fn conn ->\n      Plug.Conn.send_resp(conn, 200, \"hello\")\n    end\n\n> #### Plug dependency {: .info}\n>\n> In order to use this feature, you need to add `:plug` as a dependency.","ref":"Kino.Proxy.html"},{"type":"module","title":"Examples - Kino.Proxy","doc":"Using the proxy feature, we can use Livebook apps to build APIs.\nFor example, we could provide a data export endpoint:\n\n\n    Kino.Proxy.listen(fn\n      %{path_info: [\"export\", \"data\"]} = conn ->\n        data = \"some data\"\n\n        conn\n        |> Plug.Conn.put_resp_header(\"content-type\", \"application/csv\")\n        |> Plug.Conn.send_resp(200, data)\n\n      conn ->\n        conn\n        |> Plug.Conn.put_resp_header(\"content-type\", \"application/text\")\n        |> Plug.Conn.send_resp(200, \"use /export/data to get extract the report data\")\n    end)\n\nOnce deployed as an app, the API client would be able to export the data\nby sending a request to `/apps/:slug/proxy/export/data`.\n\n> #### Authentication {: .warning}\n>\n> The paths exposed by `Kino.Proxy` don't use the authentication mechanisms\n> defined in your Livebook instance.\n>\n> If you need to authenticate requests, you should\n> implement your own authentication mechanism. Here's a simple example.\n>\n> ```elixir\n> Kino.Proxy.listen(fn conn ->\n>   expected_token = \"my-secret-api-token\"\n>\n>   with [\"Bearer \" <> user_token] <- Plug.Conn.get_req_header(conn, \"authorization\"),\n>        true <- Plug.Crypto.secure_compare(user_token, expected_token) do\n>     Plug.Conn.send_resp(conn, 200, \"hello\")\n>   else\n>     _ ->\n>       conn\n>       |> Plug.Conn.put_resp_header(\"www-authenticate\", \"Bearer\")\n>       |> Plug.Conn.send_resp(401, \"Unauthorized\")\n>   end\n> end)\n> ```","ref":"Kino.Proxy.html#module-examples"},{"type":"function","title":"Kino.Proxy.listen/1","doc":"Registers a request listener.\n\nExpects the listener to be a function that handles a request\n`Plug.Conn`.","ref":"Kino.Proxy.html#listen/1"},{"type":"module","title":"Kino.RPC","doc":"Functions for working with remote nodes.","ref":"Kino.RPC.html"},{"type":"macro","title":"Kino.RPC.eval_string/3","doc":"Evaluates the contents given by `string` on the given `node`.\n\nReturns the value returned from evaluation.\n\nThe code is analyzed for variable references, they are automatically\nextracted from the caller binding and passed to the evaluation. This\nmeans that the evaluated string actually has closure semantics.\n\nThe code is parsed and expanded on the remote node. Also, errors\nand exits are captured and propagated to the caller.\n\nSee `Code.eval_string/3` for available `opts`.","ref":"Kino.RPC.html#eval_string/3"},{"type":"module","title":"Kino.Shorts","doc":"Shortcuts for building Kinos.\n\nThis module provides an easy to use Kino API and is meant to\nbe imported into your notebooks:\n\n    import Kino.Shorts","ref":"Kino.Shorts.html"},{"type":"function","title":"Kino.Shorts.audio/2","doc":"Renders an audio of any given format.\n\nIt is a wrapper around `Kino.Audio.new/2`.","ref":"Kino.Shorts.html#audio/2"},{"type":"function","title":"Examples - Kino.Shorts.audio/2","doc":"import Kino.Shorts\n    content = File.read!(\"/path/to/audio.wav\")\n    audio(content, :wav)","ref":"Kino.Shorts.html#audio/2-examples"},{"type":"function","title":"Kino.Shorts.data_table/2","doc":"Renders a data table output for user-provided tabular data.\n\nThe data must implement the `Table.Reader` protocol. This\nfunction is a wrapper around `Kino.DataTable.new/1`.","ref":"Kino.Shorts.html#data_table/2"},{"type":"function","title":"Examples - Kino.Shorts.data_table/2","doc":"import Kino.Shorts\n\n    data = [\n      %{id: 1, name: \"Elixir\", website: \"https://elixir-lang.org\"},\n      %{id: 2, name: \"Erlang\", website: \"https://www.erlang.org\"}\n    ]\n\n    data_table(data)","ref":"Kino.Shorts.html#data_table/2-examples"},{"type":"function","title":"Kino.Shorts.download/2","doc":"Renders a file download button.\n\nThe given function is invoked to generate the file content whenever\na download is requested.\n\nIt is a wrapper around `Kino.Download.new/2`.","ref":"Kino.Shorts.html#download/2"},{"type":"function","title":"Options - Kino.Shorts.download/2","doc":"* `:filename` - the default filename suggested for download.\n    Defaults to `\"download\"`\n\n  * `:label` - the button text. Defaults to the value of `:filename`\n    if present and `\"Download\"` otherwise","ref":"Kino.Shorts.html#download/2-options"},{"type":"function","title":"Examples - Kino.Shorts.download/2","doc":"download(fn ->\n      \"Example text\"\n    end)\n\n    download(\n      fn -> Jason.encode!(%{\"foo\" => \"bar\"}) end,\n      filename: \"data.json\"\n    )\n\n    download(\n      fn -> <<0, 1>> end,\n      filename: \"data.bin\",\n      label: \"Binary data\"\n    )","ref":"Kino.Shorts.html#download/2-examples"},{"type":"function","title":"Kino.Shorts.frame/1","doc":"A placeholder for static outputs that can be dynamically updated.\n\nThe frame can be updated with the `Kino.Frame` module API.\nAlso see `Kino.animate/3`.","ref":"Kino.Shorts.html#frame/1"},{"type":"function","title":"Examples - Kino.Shorts.frame/1","doc":"import Kino.Shorts\n    frame = frame() |> Kino.render()\n\n    for i <- 1..100 do\n      Kino.Frame.render(frame, i)\n      Process.sleep(50)\n    end","ref":"Kino.Shorts.html#frame/1-examples"},{"type":"function","title":"Kino.Shorts.grid/2","doc":"Arranges outputs into a grid.\n\nNote that the grid does not support scrolling, it always squeezes\nthe content, such that it does not exceed the page width.\n\nIt is a wrapper around `Kino.Layout.grid/2`. See the linked function\nfor all supported options.","ref":"Kino.Shorts.html#grid/2"},{"type":"function","title":"Options - Kino.Shorts.grid/2","doc":"* `:columns` - the number of columns in the grid. Defaults to `1`\n\n  * `:boxed` - whether the grid should be wrapped in a bordered box.\n    Defaults to `false`\n\n  * `:gap` - the amount of spacing between grid items in pixels.\n    Defaults to `8`","ref":"Kino.Shorts.html#grid/2-options"},{"type":"function","title":"Examples - Kino.Shorts.grid/2","doc":"import Kino.Shorts\n\n    images =\n      for path <- paths do\n        path |> File.read!() |> image(:jpeg)\n      end\n\n    grid(images, columns: 3)","ref":"Kino.Shorts.html#grid/2-examples"},{"type":"function","title":"Kino.Shorts.html/1","doc":"Displays arbitrary static HTML.\n\nIt is a wrapper around `Kino.HTML.new/1`.","ref":"Kino.Shorts.html#html/1"},{"type":"function","title":"Examples - Kino.Shorts.html/1","doc":"import Kino.Shorts\n\n    html(\"\"\"\n     Look! \n\n     I wrote this HTML from  Kino ! \n    \"\"\")","ref":"Kino.Shorts.html#html/1-examples"},{"type":"function","title":"Kino.Shorts.image/2","doc":"Renders an image of any given format.\n\nIt is a wrapper around `Kino.Image.new/2`.","ref":"Kino.Shorts.html#image/2"},{"type":"function","title":"Examples - Kino.Shorts.image/2","doc":"import Kino.Shorts\n    content = File.read!(\"/path/to/image.jpeg\")\n    image(content, \"image/jpeg\")","ref":"Kino.Shorts.html#image/2-examples"},{"type":"function","title":"Kino.Shorts.markdown/1","doc":"Renders Markdown content, in case you need richer text.\n\nIt is a wrapper around `Kino.Markdown.new/1`.","ref":"Kino.Shorts.html#markdown/1"},{"type":"function","title":"Examples - Kino.Shorts.markdown/1","doc":"import Kino.Shorts\n\n    markdown(\"\"\"\n    # Example\n\n    A regular Markdown file.","ref":"Kino.Shorts.html#markdown/1-examples"},{"type":"function","title":"Code - Kino.Shorts.markdown/1","doc":"```elixir\n    \"Elixir\" |> String.graphemes() |> Enum.frequencies()\n    ```","ref":"Kino.Shorts.html#markdown/1-code"},{"type":"function","title":"Table - Kino.Shorts.markdown/1","doc":"| ID | Name   | Website                 |\n    | -- | ------ | ----------------------- |\n    | 1  | Elixir | https://elixir-lang.org |\n    | 2  | Erlang | https://www.erlang.org  |\n    \"\"\")","ref":"Kino.Shorts.html#markdown/1-table"},{"type":"function","title":"Kino.Shorts.mermaid/1","doc":"Renders Mermaid graphs.\n\nIt is a wrapper around `Kino.Mermaid.new/1`.","ref":"Kino.Shorts.html#mermaid/1"},{"type":"function","title":"Examples - Kino.Shorts.mermaid/1","doc":"import Kino.Shorts\n\n    mermaid(\"\"\"\n    graph TD;\n      A-->B;\n      A-->C;\n      B-->D;\n      C-->D;\n    \"\"\")","ref":"Kino.Shorts.html#mermaid/1-examples"},{"type":"function","title":"Kino.Shorts.read_audio/2","doc":"Renders and reads a new audio input.\n\nSee `Kino.Input.audio/2` for all supported formats and options.\n\n> #### Warning {: .warning}\n>\n> The audio input is shared by default: once you upload an audio,\n> the audio will be replicated to all users reading the notebook.\n> Use `Kino.Control.form/2` if you want each user to have a distinct\n> audio upload with an explicit submission button.","ref":"Kino.Shorts.html#read_audio/2"},{"type":"function","title":"Kino.Shorts.read_checkbox/2","doc":"Renders and reads a new checkbox.","ref":"Kino.Shorts.html#read_checkbox/2"},{"type":"function","title":"Options - Kino.Shorts.read_checkbox/2","doc":"* `:default` - the initial input value. Defaults to `false`","ref":"Kino.Shorts.html#read_checkbox/2-options"},{"type":"function","title":"Kino.Shorts.read_color/2","doc":"Renders and reads a new color input.","ref":"Kino.Shorts.html#read_color/2"},{"type":"function","title":"Options - Kino.Shorts.read_color/2","doc":"* `:default` - the initial input value. Defaults to `#6583FF`","ref":"Kino.Shorts.html#read_color/2-options"},{"type":"function","title":"Kino.Shorts.read_date/2","doc":"Renders and reads a new date input.","ref":"Kino.Shorts.html#read_date/2"},{"type":"function","title":"Options - Kino.Shorts.read_date/2","doc":"* `:default` - the initial input value. Defaults to `nil`\n\n  * `:min` - the minimum date value\n\n  * `:max` - the maximum date value","ref":"Kino.Shorts.html#read_date/2-options"},{"type":"function","title":"Kino.Shorts.read_file/2","doc":"Renders and reads a new file input.\n\nThe file path can then be accessed using `Kino.Input.file_path/1`.\nSee `Kino.Input.file/2` for additional considerations.\n\n> #### Warning {: .warning}\n>\n> The file input is shared by default: once you upload a file,\n> the file will be replicated to all users reading the notebook.\n> Use `Kino.Control.form/2` if you want each user to have a distinct\n> file upload with an explicit submission button.","ref":"Kino.Shorts.html#read_file/2"},{"type":"function","title":"Options - Kino.Shorts.read_file/2","doc":"* `:accept` - the list of accepted file types (either extensions\n    or MIME types) or `:any`. Defaults to `:any`","ref":"Kino.Shorts.html#read_file/2-options"},{"type":"function","title":"Kino.Shorts.read_image/2","doc":"Renders and reads a new image input.\n\nSee `Kino.Input.image/2` for all supported formats and options.\n\n> #### Warning {: .warning}\n>\n> The image input is shared by default: once you upload a image,\n> the image will be replicated to all users reading the notebook.\n> Use `Kino.Control.form/2` if you want each user to have a distinct\n> image upload with an explicit submission button.","ref":"Kino.Shorts.html#read_image/2"},{"type":"function","title":"Kino.Shorts.read_number/2","doc":"Renders and reads a new number input.","ref":"Kino.Shorts.html#read_number/2"},{"type":"function","title":"Options - Kino.Shorts.read_number/2","doc":"* `:default` - the initial input value. Defaults to `nil`","ref":"Kino.Shorts.html#read_number/2-options"},{"type":"function","title":"Kino.Shorts.read_password/2","doc":"Renders and reads a new password input.","ref":"Kino.Shorts.html#read_password/2"},{"type":"function","title":"Options - Kino.Shorts.read_password/2","doc":"* `:default` - the initial input value. Defaults to `\"\"`","ref":"Kino.Shorts.html#read_password/2-options"},{"type":"function","title":"Kino.Shorts.read_range/2","doc":"Renders and reads a new slider input.","ref":"Kino.Shorts.html#read_range/2"},{"type":"function","title":"Options - Kino.Shorts.read_range/2","doc":"* `:default` - the initial input value. Defaults to the\n    minimum value\n\n  * `:min` - the minimum value\n\n  * `:max` - the maximum value\n\n  * `:step` - the slider increment","ref":"Kino.Shorts.html#read_range/2-options"},{"type":"function","title":"Kino.Shorts.read_select/3","doc":"Renders and reads a new select input.\n\nThe input expects a list of options in the form `[{value, label}]`,\nwhere `value` is an arbitrary term and `label` is a descriptive\nstring.","ref":"Kino.Shorts.html#read_select/3"},{"type":"function","title":"Options - Kino.Shorts.read_select/3","doc":"* `:default` - the initial input value. Defaults to the first\n    value from the given list of options","ref":"Kino.Shorts.html#read_select/3-options"},{"type":"function","title":"Examples - Kino.Shorts.read_select/3","doc":"read_select(\"Language\", [en: \"English\", fr: \"Français\"])\n\n    read_select(\"Language\", [{1, \"One\"}, {2, \"Two\"}, {3, \"Three\"}])","ref":"Kino.Shorts.html#read_select/3-examples"},{"type":"function","title":"Kino.Shorts.read_text/2","doc":"Renders and reads a new text input.","ref":"Kino.Shorts.html#read_text/2"},{"type":"function","title":"Options - Kino.Shorts.read_text/2","doc":"* `:default` - the initial input value. Defaults to `\"\"`","ref":"Kino.Shorts.html#read_text/2-options"},{"type":"function","title":"Kino.Shorts.read_textarea/2","doc":"Renders and reads a new multiline text input.","ref":"Kino.Shorts.html#read_textarea/2"},{"type":"function","title":"Options - Kino.Shorts.read_textarea/2","doc":"* `:default` - the initial input value. Defaults to `\"\"`\n\n  * `:monospace` - whether to use a monospace font inside the textarea.\n    Defaults to `false`","ref":"Kino.Shorts.html#read_textarea/2-options"},{"type":"function","title":"Kino.Shorts.read_url/2","doc":"Renders and reads a new URL input.","ref":"Kino.Shorts.html#read_url/2"},{"type":"function","title":"Options - Kino.Shorts.read_url/2","doc":"* `:default` - the initial input value. Defaults to `nil`","ref":"Kino.Shorts.html#read_url/2-options"},{"type":"function","title":"Kino.Shorts.read_utc_datetime/2","doc":"Renders and reads a new datetime input.","ref":"Kino.Shorts.html#read_utc_datetime/2"},{"type":"function","title":"Options - Kino.Shorts.read_utc_datetime/2","doc":"* `:default` - the initial input value. Defaults to `nil`\n\n  * `:min` - the minimum datetime value (in UTC)\n\n  * `:max` - the maximum datetime value (in UTC)","ref":"Kino.Shorts.html#read_utc_datetime/2-options"},{"type":"function","title":"Kino.Shorts.read_utc_time/2","doc":"Renders and reads a new time input.","ref":"Kino.Shorts.html#read_utc_time/2"},{"type":"function","title":"Options - Kino.Shorts.read_utc_time/2","doc":"* `:default` - the initial input value. Defaults to `nil`\n\n  * `:min` - the minimum time value (in UTC)\n\n  * `:max` - the maximum time value (in UTC)","ref":"Kino.Shorts.html#read_utc_time/2-options"},{"type":"function","title":"Kino.Shorts.tabs/1","doc":"Arranges outputs into separate tabs.\n\nIt is a wrapper around `Kino.Layout.tabs/1`.","ref":"Kino.Shorts.html#tabs/1"},{"type":"function","title":"Examples - Kino.Shorts.tabs/1","doc":"import Kino.Shorts\n\n    data = [\n      %{id: 1, name: \"Elixir\", website: \"https://elixir-lang.org\"},\n      %{id: 2, name: \"Erlang\", website: \"https://www.erlang.org\"}\n    ]\n\n    tabs([\n      Table: data_table(data),\n      Raw: data\n    ])","ref":"Kino.Shorts.html#tabs/1-examples"},{"type":"function","title":"Kino.Shorts.text/1","doc":"Renders plain text content.\n\nIt is similar to `markdown/1`, however doesn't interpret any markup.\n\nIt is a wrapper around `Kino.Text.new/1`.","ref":"Kino.Shorts.html#text/1"},{"type":"function","title":"Examples - Kino.Shorts.text/1","doc":"import Kino.Shorts\n    text(\"Hello!\")","ref":"Kino.Shorts.html#text/1-examples"},{"type":"function","title":"Kino.Shorts.tree/1","doc":"Displays arbitrarily nested data structure as a tree view.\n\nIt is a wrapper around `Kino.Tree.new/1`.","ref":"Kino.Shorts.html#tree/1"},{"type":"function","title":"Examples - Kino.Shorts.tree/1","doc":"import Kino.Shorts\n    tree(Process.info(self()))","ref":"Kino.Shorts.html#tree/1-examples"},{"type":"function","title":"Kino.Shorts.video/2","doc":"Renders a video of any given format.\n\nIt is a wrapper around `Kino.Video.new/2`.","ref":"Kino.Shorts.html#video/2"},{"type":"function","title":"Examples - Kino.Shorts.video/2","doc":"import Kino.Shorts\n    content = File.read!(\"/path/to/video.mp4\")\n    video(content, :mp4)","ref":"Kino.Shorts.html#video/2-examples"},{"type":"module","title":"Kino.Workspace","doc":"Functions related to workspace integrations and Livebook apps.","ref":"Kino.Workspace.html"},{"type":"function","title":"Kino.Workspace.app_info/0","doc":"Returns information about the running app.\n\nNote that `:started_by` information is only available for multi-session\napps when the app uses a Livebook Teams workspace.\n\nUnless called from within an app deployment, returns `%{type: :none}`.","ref":"Kino.Workspace.html#app_info/0"},{"type":"function","title":"Kino.Workspace.user_info/1","doc":"Returns user information for the given connected client id.\n\nNote that this information is only available when the session uses\nLivebook Teams workspace, otherwise `:not_available` error is returned.\n\nIf there is no such connected client, `:not_found` error is returned.","ref":"Kino.Workspace.html#user_info/1"},{"type":"type","title":"Kino.Workspace.app_info/0","doc":"","ref":"Kino.Workspace.html#t:app_info/0"},{"type":"type","title":"Kino.Workspace.user_info/0","doc":"","ref":"Kino.Workspace.html#t:user_info/0"},{"type":"module","title":"Kino.Audio","doc":"A kino for rendering a binary audio.","ref":"Kino.Audio.html"},{"type":"module","title":"Examples - Kino.Audio","doc":"content = File.read!(\"/path/to/audio.wav\")\n    Kino.Audio.new(content, :wav)\n\n    content = File.read!(\"/path/to/audio.wav\")\n    Kino.Audio.new(content, :wav, autoplay: true, loop: true)","ref":"Kino.Audio.html#module-examples"},{"type":"function","title":"Kino.Audio.new/3","doc":"Creates a new kino displaying the given binary audio.\n\nThe given type be either `:wav`, `:mp3`/`:mpeg`, `:ogg`\nor a string with audio MIME type.","ref":"Kino.Audio.html#new/3"},{"type":"function","title":"Options - Kino.Audio.new/3","doc":"* `:autoplay` - whether the audio should start playing as soon as\n    it is rendered. Defaults to `false`\n\n  * `:loop` - whether the audio should loop. Defaults to `false`\n\n  * `:muted` - whether the audio should be muted. Defaults to `false`","ref":"Kino.Audio.html#new/3-options"},{"type":"type","title":"Kino.Audio.common_audio_type/0","doc":"","ref":"Kino.Audio.html#t:common_audio_type/0"},{"type":"type","title":"Kino.Audio.mime_type/0","doc":"","ref":"Kino.Audio.html#t:mime_type/0"},{"type":"type","title":"Kino.Audio.t/0","doc":"","ref":"Kino.Audio.html#t:t/0"},{"type":"module","title":"Kino.DataTable","doc":"A kino for interactively viewing tabular data.","ref":"Kino.DataTable.html"},{"type":"module","title":"Examples - Kino.DataTable","doc":"data = [\n      %{id: 1, name: \"Elixir\", website: \"https://elixir-lang.org\"},\n      %{id: 2, name: \"Erlang\", website: \"https://www.erlang.org\"}\n    ]\n\n    Kino.DataTable.new(data)\n\nThe tabular view allows you to quickly preview the data\nand analyze it thanks to sorting capabilities.\n\n    data =\n      for pid <- Process.list() do\n        pid |> Process.info() |> Keyword.merge(registered_name: nil)\n      end\n\n    Kino.DataTable.new(\n      data,\n      keys: [:registered_name, :initial_call, :reductions, :stack_size]\n    )","ref":"Kino.DataTable.html#module-examples"},{"type":"function","title":"Kino.DataTable.new/2","doc":"Creates a new kino displaying given tabular data.","ref":"Kino.DataTable.html#new/2"},{"type":"function","title":"Options - Kino.DataTable.new/2","doc":"* `:keys` - a list of keys to include in the table for each record.\n    The order is reflected in the rendered table. Optional\n\n  * `:name` - The displayed name of the table. Defaults to `\"Data\"`\n\n  * `:sorting_enabled` - whether the table should support sorting the\n    data. Sorting requires traversal of the whole enumerable, so it\n    may not be desirable for large lazy enumerables. Defaults to `true`\n\n * `:formatter` - a 2-arity function that is used to format the data\n   in the table. The first parameter passed is the `key` (column name) and\n   the second is the value to be formatted. When formatting column headings\n   the key is the special value `:__header__`. The formatter function must\n   return either `{:ok, string}` or `:default`. When the return value is\n   `:default` the default data table formatting is applied.","ref":"Kino.DataTable.html#new/2-options"},{"type":"function","title":"Kino.DataTable.update/3","doc":"Updates the table to display a new tabular data.","ref":"Kino.DataTable.html#update/3"},{"type":"function","title":"Options - Kino.DataTable.update/3","doc":"* `:keys` - a list of keys to include in the table for each record.\n    The order is reflected in the rendered table. Optional","ref":"Kino.DataTable.html#update/3-options"},{"type":"function","title":"Examples - Kino.DataTable.update/3","doc":"data = [\n      %{id: 1, name: \"Elixir\", website: \"https://elixir-lang.org\"},\n      %{id: 2, name: \"Erlang\", website: \"https://www.erlang.org\"}\n    ]\n\n    kino = Kino.DataTable.new(data)\n\nOnce created, you can update the table to display new data:\n\n    new_data = [\n      %{id: 1, name: \"Elixir Lang\", website: \"https://elixir-lang.org\"},\n      %{id: 2, name: \"Erlang Lang\", website: \"https://www.erlang.org\"}\n    ]\n\n    Kino.DataTable.update(kino, new_data)","ref":"Kino.DataTable.html#update/3-examples"},{"type":"type","title":"Kino.DataTable.t/0","doc":"","ref":"Kino.DataTable.html#t:t/0"},{"type":"module","title":"Kino.Download","doc":"A kino for downloading file content.","ref":"Kino.Download.html"},{"type":"module","title":"Examples - Kino.Download","doc":"Kino.Download.new(fn ->\n      \"Example text\"\n    end)\n\n    Kino.Download.new(\n      fn -> Jason.encode!(%{\"foo\" => \"bar\"}) end,\n      filename: \"data.json\"\n    )\n\n    Kino.Download.new(\n      fn -> <<0, 1>> end,\n      filename: \"data.bin\",\n      label: \"Binary data\"\n    )","ref":"Kino.Download.html#module-examples"},{"type":"function","title":"Kino.Download.new/2","doc":"Creates a button for file download.\n\nThe given function is invoked to generate the file content whenever\na download is requested.","ref":"Kino.Download.html#new/2"},{"type":"function","title":"Options - Kino.Download.new/2","doc":"* `:filename` - the default filename suggested for download.\n    Defaults to `\"download\"`\n\n  * `:label` - the button text. Defaults to the value of `:filename`\n    if present and `\"Download\"` otherwise","ref":"Kino.Download.html#new/2-options"},{"type":"type","title":"Kino.Download.t/0","doc":"","ref":"Kino.Download.html#t:t/0"},{"type":"module","title":"Kino.ETS","doc":"A kino for interactively viewing an ETS table.","ref":"Kino.ETS.html"},{"type":"module","title":"Examples - Kino.ETS","doc":"tid = :ets.new(:users, [:set, :public])\n    Kino.ETS.new(tid)\n\n    Kino.ETS.new(:elixir_config)","ref":"Kino.ETS.html#module-examples"},{"type":"function","title":"Kino.ETS.new/1","doc":"Creates a new kino displaying the given ETS table.\n\nNote that private tables cannot be read by an arbitrary process,\nso the given table must have either public or protected access.","ref":"Kino.ETS.html#new/1"},{"type":"type","title":"Kino.ETS.t/0","doc":"","ref":"Kino.ETS.html#t:t/0"},{"type":"module","title":"Kino.HTML","doc":"A kino for rendering HTML content.\n\nThe HTML may include ` ` tags with global JS to be executed.\n\nIn case you need to parameterize the HTML with dynamic values, write\na custom `Kino.JS` component.","ref":"Kino.HTML.html"},{"type":"module","title":"Examples - Kino.HTML","doc":"Kino.HTML.new(\"\"\"\n     Look! \n\n     I wrote this HTML from  Kino ! \n    \"\"\")\n\n    Kino.HTML.new(\"\"\"\n     Click \n\n     \n      const button = document.querySelector(\"#button\");\n\n      button.addEventListener(\"click\", (event) => {\n        button.textContent = \"Clicked!\"\n      });\n     \n    \"\"\")","ref":"Kino.HTML.html#module-examples"},{"type":"function","title":"Kino.HTML.new/1","doc":"Creates a new kino displaying the given HTML.","ref":"Kino.HTML.html#new/1"},{"type":"type","title":"Kino.HTML.t/0","doc":"","ref":"Kino.HTML.html#t:t/0"},{"type":"module","title":"Kino.Image","doc":"A kino for rendering a binary image.\n\nThis is just a meta-struct that implements the `Kino.Render`\nprotocol, so that it gets rendered as the underlying image.","ref":"Kino.Image.html"},{"type":"module","title":"Examples - Kino.Image","doc":"content = File.read!(\"/path/to/image.jpeg\")\n    Kino.Image.new(content, \"image/jpeg\")","ref":"Kino.Image.html#module-examples"},{"type":"function","title":"Kino.Image.new/1","doc":"Creates a new kino similarly to `new/2` from a compatible term.\n\nCurrently the supported terms are:\n\n  * `Nx.Tensor` in HWC order","ref":"Kino.Image.html#new/1"},{"type":"function","title":"Kino.Image.new/2","doc":"Creates a new kino displaying the given binary image.\n\nThe given type be either `:jpeg`/`:jpg`, `:png`, `:gif`, `:svg`, `:pixel`\nor a string with image MIME type.","ref":"Kino.Image.html#new/2"},{"type":"function","title":"Pixel data - Kino.Image.new/2","doc":"Note that a special `image/x-pixel` MIME type is supported. The\nbinary consists of the following consecutive parts:\n\n  * height - 32 bits (unsigned big-endian integer)\n  * width - 32 bits (unsigned big-endian integer)\n  * channels - 8 bits (unsigned integer)\n  * data - pixel data in HWC order\n\nPixel data consists of 8-bit unsigned integers. The number of channels\ncan be either: 1 (grayscale), 2 (grayscale + alpha), 3 (RGB), or 4\n(RGB + alpha).","ref":"Kino.Image.html#new/2-pixel-data"},{"type":"type","title":"Kino.Image.common_image_type/0","doc":"","ref":"Kino.Image.html#t:common_image_type/0"},{"type":"type","title":"Kino.Image.mime_type/0","doc":"","ref":"Kino.Image.html#t:mime_type/0"},{"type":"opaque","title":"Kino.Image.t/0","doc":"","ref":"Kino.Image.html#t:t/0"},{"type":"module","title":"Kino.Layout","doc":"Layout utilities for arranging multiple kinos together.","ref":"Kino.Layout.html"},{"type":"function","title":"Kino.Layout.grid/2","doc":"Arranges outputs into a grid.\n\nNote that the grid does not support scrolling, it always squeezes\nthe content, such that it does not exceed the page width.","ref":"Kino.Layout.html#grid/2"},{"type":"function","title":"Options - Kino.Layout.grid/2","doc":"* `:columns` - the number of columns in the grid. Defaults to `1`\n\n  * `:boxed` - whether the grid should be wrapped in a bordered box.\n    Defaults to `false`\n\n  * `:gap` - the amount of spacing between grid items in pixels.\n    Defaults to `8`","ref":"Kino.Layout.html#grid/2-options"},{"type":"function","title":"Examples - Kino.Layout.grid/2","doc":"images =\n      for path <- paths do\n        path |> File.read!() |> Kino.Image.new(:jpeg)\n      end\n\n    Kino.Layout.grid(images, columns: 3)","ref":"Kino.Layout.html#grid/2-examples"},{"type":"function","title":"Kino.Layout.tabs/1","doc":"Arranges outputs into separate tabs.","ref":"Kino.Layout.html#tabs/1"},{"type":"function","title":"Examples - Kino.Layout.tabs/1","doc":"data = [\n      %{id: 1, name: \"Elixir\", website: \"https://elixir-lang.org\"},\n      %{id: 2, name: \"Erlang\", website: \"https://www.erlang.org\"}\n    ]\n\n    Kino.Layout.tabs([\n      Table: Kino.DataTable.new(data),\n      Raw: data\n    ])","ref":"Kino.Layout.html#tabs/1-examples"},{"type":"opaque","title":"Kino.Layout.t/0","doc":"","ref":"Kino.Layout.html#t:t/0"},{"type":"module","title":"Kino.Markdown","doc":"A kino for rendering Markdown content.\n\nThis is just a meta-struct that implements the `Kino.Render`\nprotocol, so that it gets rendered as markdown.","ref":"Kino.Markdown.html"},{"type":"module","title":"Examples - Kino.Markdown","doc":"Kino.Markdown.new(\"\"\"\n    # Example\n\n    A regular Markdown file.","ref":"Kino.Markdown.html#module-examples"},{"type":"module","title":"Code - Kino.Markdown","doc":"```elixir\n    \"Elixir\" |> String.graphemes() |> Enum.frequencies()\n    ```","ref":"Kino.Markdown.html#module-code"},{"type":"module","title":"Table - Kino.Markdown","doc":"| ID | Name   | Website                 |\n    | -- | ------ | ----------------------- |\n    | 1  | Elixir | https://elixir-lang.org |\n    | 2  | Erlang | https://www.erlang.org  |\n    \"\"\")\n\nThis format may come in handy when exploring Markdown\nfrom external sources:\n\n    text = File.read!(\"/path/to/README.md\")\n    Kino.Markdown.new(text)","ref":"Kino.Markdown.html#module-table"},{"type":"function","title":"Kino.Markdown.new/2","doc":"Creates a new kino displaying the given Markdown content.","ref":"Kino.Markdown.html#new/2"},{"type":"function","title":"Options - Kino.Markdown.new/2","doc":"* `:chunk` - whether this is a part of a larger text. Adjacent chunks\n    are merged into a single text. This is useful for streaming content.\n    Defaults to `false`","ref":"Kino.Markdown.html#new/2-options"},{"type":"function","title":"Examples - Kino.Markdown.new/2","doc":"","ref":"Kino.Markdown.html#new/2-examples"},{"type":"function","title":"Using the `:chunk` option - Kino.Markdown.new/2","doc":"Using a `Kino.Frame`.\n\n    frame = Kino.Frame.new() |> Kino.render()\n\n    for word <- [\"who\", \" *let*\", \" `the`\", \" **dogs**\", \" out\"] do\n      text = Kino.Markdown.new(word, chunk: true)\n      Kino.Frame.append(frame, text)\n      Process.sleep(250)\n    end\n\nWithout using a `Kino.Frame`.\n\n    for word <- [\"who\", \" *let*\", \" `the`\", \" **dogs**\", \" out\"] do\n      Kino.Markdown.new(word, chunk: true) |> Kino.render()\n      Process.sleep(250)\n    end\n\n    Kino.nothing()","ref":"Kino.Markdown.html#new/2-using-the-chunk-option"},{"type":"opaque","title":"Kino.Markdown.t/0","doc":"","ref":"Kino.Markdown.html#t:t/0"},{"type":"module","title":"Kino.Mermaid","doc":"A kino for rendering Mermaid graphs.\n\n> #### Relation to Kino.Markdown {: .info}\n>\n> Mermaid graphs can also be generated dynamically with `Kino.Markdown`,\n> however the output of `Kino.Markdown` is never persisted in the\n> notebook source. `Kino.Mermaid` doesn't have this limitation.","ref":"Kino.Mermaid.html"},{"type":"module","title":"Examples - Kino.Mermaid","doc":"Kino.Mermaid.new(\"\"\"\n    graph TD;\n      A-->B;\n      A-->C;\n      B-->D;\n      C-->D;\n    \"\"\")","ref":"Kino.Mermaid.html#module-examples"},{"type":"function","title":"Kino.Mermaid.new/1","doc":"Creates a new kino displaying the given Mermaid graph.","ref":"Kino.Mermaid.html#new/1"},{"type":"type","title":"Kino.Mermaid.t/0","doc":"","ref":"Kino.Mermaid.html#t:t/0"},{"type":"module","title":"Kino.Text","doc":"A kino for rendering text content.\n\nFor rich text, use `Kino.Markdown`.","ref":"Kino.Text.html"},{"type":"module","title":"Examples - Kino.Text","doc":"Kino.Text.new(\"Hello!\")\n\n    [:green, \"Hello!\"]\n    |> IO.ANSI.format()\n    |> IO.iodata_to_binary()\n    |> Kino.Text.new(terminal: true)","ref":"Kino.Text.html#module-examples"},{"type":"function","title":"Kino.Text.new/2","doc":"Creates a new kino displaying the given text content.","ref":"Kino.Text.html#new/2"},{"type":"function","title":"Options - Kino.Text.new/2","doc":"* `:terminal` - whether to render the text as if it were printed to\n    standard output, supporting ANSI escape codes. Defaults to `false`\n\n  * `:chunk` - whether this is a part of a larger text. Adjacent chunks\n    are merged into a single text. This is useful for streaming content.\n    Defaults to `false`","ref":"Kino.Text.html#new/2-options"},{"type":"function","title":"Examples - Kino.Text.new/2","doc":"","ref":"Kino.Text.html#new/2-examples"},{"type":"function","title":"Using the `:chunk` option - Kino.Text.new/2","doc":"Using a `Kino.Frame`.\n\n    frame = Kino.Frame.new() |> Kino.render()\n\n    for word <- [\"who\", \" let\", \" the\", \" dogs\", \" out\"] do\n      text = Kino.Text.new(word, chunk: true)\n      Kino.Frame.append(frame, text)\n      Process.sleep(250)\n    end\n\nWithout using a `Kino.Frame`.\n\n    for word <- [\"who\", \" let\", \" the\", \" dogs\", \" out\"] do\n      Kino.Text.new(word, chunk: true) |> Kino.render()\n      Process.sleep(250)\n    end\n\n    Kino.nothing()","ref":"Kino.Text.html#new/2-using-the-chunk-option"},{"type":"opaque","title":"Kino.Text.t/0","doc":"","ref":"Kino.Text.html#t:t/0"},{"type":"module","title":"Kino.Tree","doc":"A kino for interactively viewing nested data as a tree view.\n\nThe data can be any term.","ref":"Kino.Tree.html"},{"type":"module","title":"Examples - Kino.Tree","doc":"data = %{\n      id: 1,\n      email: \"user@example.com\",\n      inserted_at: ~U[2022-01-01T10:00:00Z],\n      addresses: [\n        %{\n          country: \"pl\",\n          city: \"Kraków\",\n          street: \"Karmelicka\",\n          zip: \"00123\"\n        }\n      ]\n    }\n\n    Kino.Tree.new(data)\n\nThe tree view is particularly useful when inspecting larger data\nstructures:\n\n    data = Process.info(self())\n    Kino.Tree.new(data)","ref":"Kino.Tree.html#module-examples"},{"type":"function","title":"Kino.Tree.new/1","doc":"Creates a new kino displaying the given data structure.","ref":"Kino.Tree.html#new/1"},{"type":"module","title":"Kino.Video","doc":"A kino for rendering a binary video.","ref":"Kino.Video.html"},{"type":"module","title":"Examples - Kino.Video","doc":"content = File.read!(\"/path/to/video.mp4\")\n    Kino.Video.new(content, :mp4)\n\n    content = File.read!(\"/path/to/video.mp4\")\n    Kino.Video.new(content, :mp4, autoplay: true, loop: true)","ref":"Kino.Video.html#module-examples"},{"type":"function","title":"Kino.Video.new/3","doc":"Creates a new kino displaying the given binary video.\n\nThe given type be either `:mp4`, `:ogg`, `:avi`, `:wmv`, `:mov`\nor a string with video MIME type.","ref":"Kino.Video.html#new/3"},{"type":"function","title":"Options - Kino.Video.new/3","doc":"* `:autoplay` - whether the video should start playing as soon as\n    it is rendered. Defaults to `false`\n\n  * `:loop` - whether the video should loop. Defaults to `false`\n\n  * `:muted` - whether the video should be muted. Defaults to `false`","ref":"Kino.Video.html#new/3-options"},{"type":"type","title":"Kino.Video.common_video_type/0","doc":"","ref":"Kino.Video.html#t:common_video_type/0"},{"type":"type","title":"Kino.Video.mime_type/0","doc":"","ref":"Kino.Video.html#t:mime_type/0"},{"type":"type","title":"Kino.Video.t/0","doc":"","ref":"Kino.Video.html#t:t/0"},{"type":"module","title":"Kino.JS","doc":"Allows for defining custom JavaScript powered kinos.","ref":"Kino.JS.html"},{"type":"module","title":"Example - Kino.JS","doc":"Here's how we could define a minimal kino that embeds the given\nHTML directly into the page.\n\n    defmodule KinoDocs.HTML do\n      use Kino.JS\n\n      def new(html) do\n        Kino.JS.new(__MODULE__, html)\n      end\n\n      asset \"main.js\" do\n        \"\"\"\n        export function init(ctx, html) {\n          ctx.root.innerHTML = html;\n        }\n        \"\"\"\n      end\n    end\n\nLet's break down the API.\n\nTo define a custom kino we need to create a new module. In this\ncase we go with `KinoDocs.HTML`.\n\nWe start by adding `use Kino.JS`, which makes our module\nasset-aware. In particular, it allows us to use the `asset/2`\nmacro to define arbitrary files directly in the module source.\n\nAll kinos require a `main.js` file that defines a JavaScript\nmodule and becomes the entrypoint on the client side. The\nJavaScript module is expected to export the `init(ctx, data)`\nfunction, where `ctx` is a special object (discussed in\ndetail later) and `data` is the kino data passed from the\nElixir side. In our example the `init` function accesses the\nroot element with `ctx.root` and overrides its content with\nthe given HTML string.\n\nFinally, we define the `new(html)` function that creates kinos\nwith the given HTML. Underneath we call `Kino.JS.new/2`\nspecifying our module as the kino type and passing the data\n(available in the JavaScript `init` function later). Again,\nit's a convention for each kino module to define a `new`\nfunction to provide uniform experience for the end user.","ref":"Kino.JS.html#module-example"},{"type":"module","title":"Assets - Kino.JS","doc":"We already saw how to define a JavaScript (or any other) file\nusing the `asset/2` macro, however in most cases it's preferable\nto put assets in a dedicated directory to benefit from syntax\nhighlighting and other editor features. To do that, we just need\nto specify where the corresponding directory is located:\n\n    use Kino.JS, assets_path: \"lib/assets/html\"\n\nThe default entrypoint file is `main.js`, however you can override\nit by setting the `:entrypoint` option. The entrypoint must be a\npath relative to the assets path.","ref":"Kino.JS.html#module-assets"},{"type":"module","title":"Stylesheets - Kino.JS","doc":"The `ctx.importCSS(url)` function allows us to load CSS from the given\nURL into the page. The stylesheet can be an external resource, such as\na font from Google Fonts or a custom asset (as outlined above). Here's\nan example of both:\n\n    defmodule KinoDocs.HTML do\n      use Kino.JS\n\n      def new(html) do\n        Kino.JS.new(__MODULE__, html)\n      end\n\n      asset \"main.js\" do\n        \"\"\"\n        export function init(ctx, html) {\n          ctx.importCSS(\"https://fonts.googleapis.com/css?family=Sofia\")\n          ctx.importCSS(\"main.css\")\n\n          ctx.root.innerHTML = html;\n        }\n        \"\"\"\n      end\n\n      asset \"main.css\" do\n        \"\"\"\n        body {\n          font-family: \"Sofia\", sans-serif;\n        }\n        \"\"\"\n      end\n    end","ref":"Kino.JS.html#module-stylesheets"},{"type":"module","title":"URLs - Kino.JS","doc":"When using multiple asset files, make sure to use relative URLs.\nFor example, when adding an image to the page, instead of:\n\n     \n\nDo:\n\n     \n\nThis will correctly point to the `images/cat.jpeg` file in your\nassets.","ref":"Kino.JS.html#module-urls"},{"type":"module","title":"Security - Kino.JS","doc":"Note that all assets are assumed public and Livebook doesn't\nenforce authentication when loading them. Therefore, never\ninclude any sensitive credentials in the assets source, instead\npass them as arguments from your Elixir code.","ref":"Kino.JS.html#module-security"},{"type":"module","title":"JavaScript API - Kino.JS","doc":"In the example we briefly introduced the `ctx` (context) object\nthat is made available in the `init(ctx, data)` function. This\nobject encapsulates all of the Livebook-specific API that we can\ncall on the JavaScript side.","ref":"Kino.JS.html#module-javascript-api"},{"type":"module","title":"Properties - Kino.JS","doc":"* `ctx.root` - the root element controlled by the kino","ref":"Kino.JS.html#module-properties"},{"type":"module","title":"Functions - Kino.JS","doc":"* `ctx.importCSS(url)` - loads CSS from the given URL into the\n    page. Returns a `Promise` that resolves once the CSS is loaded\n\n  * `ctx.importJS(url)` - loads JS from the given URL into the page\n    using a regular ` ` tag. Returns a `Promise` that resolves\n    once the JS is loaded\n\n  * `ctx.handleEvent(event, callback)` - registers an event\n    handler. Once `event` is broadcasted, `callback` is executed\n    with the event payload. This applies to `Kino.JS.Live` kinos\n\n  * `ctx.pushEvent(event, payload)` - sends an event to the kino\n    server, where it is handled with `c:Kino.JS.Live.handle_event/3`.\n    This applies to `Kino.JS.Live` kinos\n\n  * `ctx.handleSync(callback)` - registers a synchronization handler,\n    it should flush any deferred UI changes to the server. This\n    applies to `Kino.SmartCell` cells\n\n  * `ctx.selectSecret(callback, preselectName)` - asks the user to\n    select a Livebook secret. Suggests `preselectName` as the default\n    choice. When the user selects a secret, `callback` is called\n    with the secret name","ref":"Kino.JS.html#module-functions"},{"type":"module","title":"Dependencies - Kino.JS","doc":"On the JavaScript side you are free to use any external packages and\nbundling tooling, as long as you provide the `main.js` file with the\n`init(ctx, data)` entrypoing. Kino itself defines a couple components\nusing `Kino.JS` and we use [esbuild](https://esbuild.github.io) to\nbundle their assets, but it's entirely up to you.\n\nFor simple components that don't require additional dependencies,\nit may be totally fine to write a single JS/CSS file without any\nbundling. Theoretically, you could even import dependencies from a\nCDN, however, we do recommend bundling dependencies with your assets\nbecause: (a) occasionally content from CDNs may get blocked; (b) most\nusers run Livebook locally, so fetching assets from the local server\nis actually faster than fetching from a CDN; (c) nowadays many packages\nactually assume their end users use a bundler.\n\nTo give a concrete example, let's say we want to render a graph using\n`mermaid`. We would define an NPM project at `assets/mermaid`, with\nregular `package.json` and the following `main.js` file:\n\n```javascript\nimport mermaid from \"mermaid\";\n\nmermaid.initialize({ startOnLoad: false });\n\nexport function init(ctx, graph) {\n  mermaid.render(\"graph1\", graph, (svgSource, bindListeners) => {\n    ctx.root.innerHTML = svgSource;\n    bindListeners && bindListeners(ctx.root);\n  });\n}\n```\n\nNext, we would bundle the file into `lib/assets/mermaid/build/main.js`,\nand reference in our Elixir module:\n\n    defmodule KinoDocs.Mermaid do\n      use Kino.JS\n\n      use Kino.JS, assets_path: \"lib/assets/mermaid/build\"\n\n      def new(graph) do\n        Kino.JS.new(__MODULE__, graph)\n      end\n    end\n\nWith all that, we would use the component like so:\n\n    KinoDocs.Mermaid.new(\"\"\"\n    graph TD;\n      A-->B;\n      A-->C;\n      B-->D;\n      C-->D;\n    \"\"\")\n\n> #### Directory structure {: .info}\n>\n> Note that we intentionally suggest keeping the NPM project in the\n> `assets/` directory, but placing the bundle output in `lib/assets/`.\n> This convention ensures that you do not include the assets source\n> (including `node_modules/`) in the Hex package, but you do include\n> the bundled assets. While it is possible to specify which directories\n> are published to Hex, following the convention makes everything work\n> as expected by default.","ref":"Kino.JS.html#module-dependencies"},{"type":"module","title":"Live kinos - Kino.JS","doc":"So far we covered the API for defining static kinos, where the\nJavaScript side only receives the initial data and there is no\nfurther interaction with the Elixir side. To introduce such\ninteraction, see `Kino.JS.Live` as a next step in our discussion.","ref":"Kino.JS.html#module-live-kinos"},{"type":"macro","title":"Kino.JS.asset/2","doc":"Defines an asset file.\n\nThis serves as a convenience when prototyping or building simple\nkinos, otherwise you most likely want to put assets in separate\nfiles. See the [Assets](#module-assets) for more details.","ref":"Kino.JS.html#asset/2"},{"type":"macro","title":"Examples - Kino.JS.asset/2","doc":"asset \"main.js\" do\n      \"\"\"\n      export function init(ctx, data) {\n        ...\n      }\n      \"\"\"\n    end\n\n    asset \"main.css\" do\n      \"\"\"\n      .box {\n        ...\n      }\n      \"\"\"\n    end","ref":"Kino.JS.html#asset/2-examples"},{"type":"function","title":"Kino.JS.new/3","doc":"Instantiates a static JavaScript kino defined by `module`.\n\nThe given `data` is passed directly to the JavaScript side during\ninitialization.","ref":"Kino.JS.html#new/3"},{"type":"function","title":"Options - Kino.JS.new/3","doc":"* `:export` - a function called to export the given kino to Markdown.\n    See the \"Export\" section below","ref":"Kino.JS.html#new/3-options"},{"type":"function","title":"Export - Kino.JS.new/3","doc":"The output can optionally be exported in notebook source by specifying\nan `:export` function. The function receives `data` as an argument\nand should return a tuple `{info_string, payload}`. `info_string`\nis used to annotate the Markdown code block where the output is\npersisted. `payload` is the value persisted in the code block. The\nvalue is automatically serialized to JSON, unless it is already a\nstring.\n\nFor example:\n\n    data = \"graph TD;A-->B;\"\n    Kino.JS.new(__MODULE__, data, export: fn data -> {\"mermaid\", data} end)\n\nWould be rendered as the following Live Markdown:\n\n````markdown\n```mermaid\ngraph TD;A-->B;\n```\n````\n\n> #### Export function {: .info}\n>\n> You should prefer to use the `data` argument for computing the\n> export payload. However, if it cannot be inferred from `data`,\n> you should just reference the original value. Do not put additional\n> fields in `data`, just to use it for export.","ref":"Kino.JS.html#new/3-export"},{"type":"opaque","title":"Kino.JS.t/0","doc":"","ref":"Kino.JS.html#t:t/0"},{"type":"behaviour","title":"Kino.JS.Live","doc":"Introduces state and event-driven capabilities to JavaScript\npowered kinos.\n\nMake sure to read the introduction to JavaScript kinos in\n`Kino.JS` for more context.\n\nSimilarly to static kinos, live kinos involve a custom JavaScript\ncode running in the browser. In fact, this part of the API is the\nsame. In addition, each live kino has a server process running on\nthe Elixir side, responsible for maintaining state and able to\ncommunicate with the JavaScript side at any time. Again, to illustrate\nthe ideas we start with a minimal example.","ref":"Kino.JS.Live.html"},{"type":"behaviour","title":"Example - Kino.JS.Live","doc":"We will follow up on our `KinoDocs.HTML` example by adding support\nfor replacing the content on demand.\n\n    defmodule KinoDocs.LiveHTML do\n      use Kino.JS\n      use Kino.JS.Live\n\n      def new(html) do\n        Kino.JS.Live.new(__MODULE__, html)\n      end\n\n      def replace(kino, html) do\n        Kino.JS.Live.cast(kino, {:replace, html})\n      end\n\n      @impl true\n      def init(html, ctx) do\n        {:ok, assign(ctx, html: html)}\n      end\n\n      @impl true\n      def handle_connect(ctx) do\n        {:ok, ctx.assigns.html, ctx}\n      end\n\n      @impl true\n      def handle_cast({:replace, html}, ctx) do\n        broadcast_event(ctx, \"replace\", html)\n        {:noreply, assign(ctx, html: html)}\n      end\n\n      asset \"main.js\" do\n        \"\"\"\n        export function init(ctx, html) {\n          ctx.root.innerHTML = html;\n\n          ctx.handleEvent(\"replace\", (html) => {\n            ctx.root.innerHTML = html;\n          });\n        }\n        \"\"\"\n      end\n    end\n\nJust as before we define a module, this time calling it\n`KinoDocs.LiveHTML` for clarity. Note many similarities to\nthe previous version, we still call `use Kino.JS`, define\nthe `main.js` file and define the `new(html)` function for\ncreating a kino instance. As a matter of fact, the initial\nresult of `KinoDocs.LiveHTML.new(html)` will render exactly\nthe same as our previous `KinoDocs.HTML.new(html)`.\n\nAs for the new bits, we added `use Kino.JS.Live` to define\na live kino server. We use `Kino.JS.Live.new/2` for creating\nthe kino instance and we implement a few `GenServer`-like\ncallbacks.\n\nOnce the kino server is started with `Kino.JS.Live.new/2`,\nthe `c:init/2` callback is called with the initial argument.\nIn this case we store the given `html` in server state.\n\nWhenever the kino is rendered on a new client, the `c:handle_connect/1`\ncallback is called and it builds the initial data for the\nclient. In this case, we always return the stored `html`.\nThis initial data is then passed to the JavaScript `init`\nfunction. Keep in mind that while the server is initialized\nonce, connect may happen at any point, as the users join/refresh\nthe page.\n\nFinally, the whole point of our example is the ability to\nreplace the HTML content directly from the Elixir side and\nfor this purpose we added the public `replace(kino, html)`\nfunction. Underneath the function uses `cast/2` to message\nour server and the message is handled with `c:handle_cast/2`.\nIn this case we store the new `html` in the server state and\nbroadcast an event with the new value. On the client side,\nwe subscribe to those events with `ctx.handleEvent(event, callback)`\nto update the page accordingly.","ref":"Kino.JS.Live.html#module-example"},{"type":"behaviour","title":"Event handlers - Kino.JS.Live","doc":"You must eventually register JavaScript handlers for all events\nthat the client may receive. However, the registration can be\ndeferred, if the initialization is asynchronous. For example,\nthe following is perfectly fine:\n\n```js\nexport function init(ctx, data) {\n  fetch(data.someUrl).then((resp) => {\n    ctx.handleEvent(\"update\", (payload) => {\n      // ...\n    });\n  });\n}\n```\n\nOr alternatively:\n\n```js\nexport async function init(ctx, data) {\n  const response = await fetch(data.someUrl);\n\n  ctx.handleEvent(\"update\", (payload) => {\n    // ...\n  });\n}\n```\n\nIn such case all incoming events are buffered and dispatched once\nthe handler is registered.","ref":"Kino.JS.Live.html#module-event-handlers"},{"type":"behaviour","title":"Binary payloads - Kino.JS.Live","doc":"The client-server communication supports binary data, both on\ninitialization and on custom events. On the server side, a binary\npayload has the form of `{:binary, info, binary}`, where `info`\nis regular JSON-serializable data that can be sent alongside\nthe plain binary.\n\nOn the client side, a binary payload is represented as `[info, buffer]`,\nwhere `info` is the additional data and `buffer` is the binary\nas `ArrayBuffer`.\n\nThe following example showcases how to send and receive events\nwith binary payloads.\n\n    defmodule KinoDocs.Binary do\n      use Kino.JS\n      use Kino.JS.Live\n\n      def new() do\n        Kino.JS.Live.new(__MODULE__, nil)\n      end\n\n      @impl true\n      def handle_connect(ctx) do\n        payload = {:binary, %{message: \"hello\"}, <<1, 2>>}\n        {:ok, payload, ctx}\n      end\n\n      @impl true\n      def handle_event(\"ping\", {:binary, _info, binary}, ctx) do\n        reply_payload = {:binary, %{message: \"pong\"}, <<1, 2, binary::binary>>}\n        broadcast_event(ctx, \"pong\", reply_payload)\n        {:noreply, ctx}\n      end\n\n      asset \"main.js\" do\n        \"\"\"\n        export function init(ctx, payload) {\n          console.log(\"initial data\", payload);\n\n          ctx.handleEvent(\"pong\", ([info, buffer]) => {\n            console.log(\"event data\", [info, buffer])\n          });\n\n          const buffer = new ArrayBuffer(2);\n          const bytes = new Uint8Array(buffer);\n          bytes[0] = 4;\n          bytes[1] = 250;\n          ctx.pushEvent(\"ping\", [{ message: \"ping\" }, buffer]);\n        }\n        \"\"\"\n      end\n    end","ref":"Kino.JS.Live.html#module-binary-payloads"},{"type":"function","title":"Kino.JS.Live.call/3","doc":"Makes a synchronous call to the kino server and waits\nfor its reply.\n\nSee `GenServer.call/3` for more details.","ref":"Kino.JS.Live.html#call/3"},{"type":"function","title":"Kino.JS.Live.cast/2","doc":"Sends an asynchronous request to the kino server.\n\nSee `GenServer.cast/2` for more details.","ref":"Kino.JS.Live.html#cast/2"},{"type":"callback","title":"Kino.JS.Live.handle_call/3","doc":"Invoked to handle synchronous `call/3` messages.\n\nSee `c:GenServer.handle_call/3` for more details.","ref":"Kino.JS.Live.html#c:handle_call/3"},{"type":"callback","title":"Kino.JS.Live.handle_cast/2","doc":"Invoked to handle asynchronous `cast/2` messages.\n\nSee `c:GenServer.handle_cast/2` for more details.","ref":"Kino.JS.Live.html#c:handle_cast/2"},{"type":"callback","title":"Kino.JS.Live.handle_connect/1","doc":"Invoked whenever a new client connects to the server.\n\nThe returned data is passed to the JavaScript `init` function\nof the connecting client.","ref":"Kino.JS.Live.html#c:handle_connect/1"},{"type":"callback","title":"Kino.JS.Live.handle_event/3","doc":"Invoked to handle client events.","ref":"Kino.JS.Live.html#c:handle_event/3"},{"type":"callback","title":"Kino.JS.Live.handle_info/2","doc":"Invoked to handle all other messages.\n\nSee `c:GenServer.handle_info/2` for more details.","ref":"Kino.JS.Live.html#c:handle_info/2"},{"type":"callback","title":"Kino.JS.Live.init/2","doc":"Invoked when the server is started.\n\nSee `c:GenServer.init/1` for more details.\n\n> #### Starting other kinos {: .warning}\n>\n> It is generally not possible to start kinos inside the `c:init/2`\n> callback, as such operation would block forever. In case you need\n> to start other kinos during initialization, you must start them\n> beforehand and pass as an argument to `c:init/2`. So instead of\n>\n>     defmodule KinoDocs.Custom do\n>       def new() do\n>         Kino.JS.Live.new(__MODULE__, {})\n>       end\n>\n>       @impl true\n>       def init({}, ctx) do\n>         frame = Kino.Frame.new()\n>         {:ok, assign(ctx, frame: frame)}\n>       end\n>\n>       ...\n>     end\n>\n> do the following\n>\n>     defmodule KinoDocs.Custom do\n>       def new() do\n>         frame = Kino.Frame.new()\n>         Kino.JS.Live.new(__MODULE__, {frame})\n>       end\n>\n>       @impl true\n>       def init({frame}, ctx) do\n>         {:ok, assign(ctx, frame: frame)}\n>       end\n>\n>       ...\n>     end\n>\n> Also see `Kino.start_child/1`.","ref":"Kino.JS.Live.html#c:init/2"},{"type":"function","title":"Kino.JS.Live.monitor/1","doc":"Starts monitoring the kino server from the calling process.\n\nRefer to `Process.monitor/1` for more details.","ref":"Kino.JS.Live.html#monitor/1"},{"type":"function","title":"Kino.JS.Live.new/3","doc":"Instantiates a live JavaScript kino defined by `module`.\n\nThe given `init_arg` is passed to the `init/2` callback when\nthe underlying kino process is started.","ref":"Kino.JS.Live.html#new/3"},{"type":"function","title":"Options - Kino.JS.Live.new/3","doc":"* `:export` - a function called to export the given kino to Markdown.\n    This works the same as `Kino.JS.new/3`, except the function\n    receives `t:Kino.JS.Live.Context.t/0` as an argument","ref":"Kino.JS.Live.html#new/3-options"},{"type":"function","title":"Kino.JS.Live.reply/2","doc":"Replies to the kino server.\n\nThis function can be used to explicitly send a reply to the kino server\nthat called `call/3` when the reply cannot be specified in the return\nvalue of `handle_call/3`.\n\nSee `GenServer.reply/2` for more details.","ref":"Kino.JS.Live.html#reply/2"},{"type":"callback","title":"Kino.JS.Live.terminate/2","doc":"Invoked when the server is about to exit.\n\nSee `c:GenServer.terminate/2` for more details.","ref":"Kino.JS.Live.html#c:terminate/2"},{"type":"type","title":"Kino.JS.Live.from/0","doc":"","ref":"Kino.JS.Live.html#t:from/0"},{"type":"type","title":"Kino.JS.Live.payload/0","doc":"","ref":"Kino.JS.Live.html#t:payload/0"},{"type":"opaque","title":"Kino.JS.Live.t/0","doc":"","ref":"Kino.JS.Live.html#t:t/0"},{"type":"module","title":"Kino.JS.Live.Context","doc":"State available in `Kino.JS.Live` server callbacks.","ref":"Kino.JS.Live.Context.html"},{"type":"module","title":"Properties - Kino.JS.Live.Context","doc":"* `:assigns` - custom server state kept across callback calls\n\n  * `:origin` - an opaque identifier of the client that triggered\n    the given action. It is set in `c:Kino.JS.Live.handle_connect/1`\n    and `c:Kino.JS.Live.handle_event/3`","ref":"Kino.JS.Live.Context.html#module-properties"},{"type":"function","title":"Kino.JS.Live.Context.assign/2","doc":"Stores key-value pairs in the state.","ref":"Kino.JS.Live.Context.html#assign/2"},{"type":"function","title":"Examples - Kino.JS.Live.Context.assign/2","doc":"assign(ctx, count: 1, timestamp: DateTime.utc_now())","ref":"Kino.JS.Live.Context.html#assign/2-examples"},{"type":"function","title":"Kino.JS.Live.Context.broadcast_event/3","doc":"Sends an event to all clients.\n\nThe event is dispatched to the registered JavaScript callback\non all connected clients.","ref":"Kino.JS.Live.Context.html#broadcast_event/3"},{"type":"function","title":"Examples - Kino.JS.Live.Context.broadcast_event/3","doc":"broadcast_event(ctx, \"new_point\", %{x: 10, y: 10})","ref":"Kino.JS.Live.Context.html#broadcast_event/3-examples"},{"type":"function","title":"Kino.JS.Live.Context.emit_event/2","doc":"Emits an event to processes subscribed to this kino.\n\nConsumers may subscribe to events emitted by the given instance of\n`Kino.JS.Live` using functions in the `Kino.Control` module, such\nas `Kino.Control.stream/1`.","ref":"Kino.JS.Live.Context.html#emit_event/2"},{"type":"function","title":"Examples - Kino.JS.Live.Context.emit_event/2","doc":"emit_event(ctx, %{event: :click, counter: 1})","ref":"Kino.JS.Live.Context.html#emit_event/2-examples"},{"type":"function","title":"Kino.JS.Live.Context.reconfigure_smart_cell/2","doc":"Updates smart cell configuration.\n\nThis function allows for re-configuring some of the options that can\nbe specified in smart cell's `c:Kino.JS.Live.init/2`.\n\nNote that this function returns the new context, which you should\nreturn from the given handler.","ref":"Kino.JS.Live.Context.html#reconfigure_smart_cell/2"},{"type":"function","title":"Options - Kino.JS.Live.Context.reconfigure_smart_cell/2","doc":"* `:editor` - note that the smart cell must be initialized with an\n    editor during on init. Supported options: `:source`, `:intellisense_node`.","ref":"Kino.JS.Live.Context.html#reconfigure_smart_cell/2-options"},{"type":"function","title":"Kino.JS.Live.Context.send_event/4","doc":"Sends an event to a specific client.\n\nThe event is dispatched to the registered JavaScript callback\non the specific connected client.","ref":"Kino.JS.Live.Context.html#send_event/4"},{"type":"function","title":"Examples - Kino.JS.Live.Context.send_event/4","doc":"send_event(ctx, origin, \"new_point\", %{x: 10, y: 10})","ref":"Kino.JS.Live.Context.html#send_event/4-examples"},{"type":"function","title":"Kino.JS.Live.Context.update/3","doc":"Updates an existing key with the given function in the state.","ref":"Kino.JS.Live.Context.html#update/3"},{"type":"function","title":"Examples - Kino.JS.Live.Context.update/3","doc":"update(ctx, :count, &(&1 + 1))","ref":"Kino.JS.Live.Context.html#update/3-examples"},{"type":"type","title":"Kino.JS.Live.Context.origin/0","doc":"","ref":"Kino.JS.Live.Context.html#t:origin/0"},{"type":"type","title":"Kino.JS.Live.Context.t/0","doc":"","ref":"Kino.JS.Live.Context.html#t:t/0"},{"type":"behaviour","title":"Kino.SmartCell","doc":"An interface for defining custom smart cells.\n\nA smart cell is a UI wizard designed for producing a piece of code\nthat accomplishes a specific task. In other words, a smart cell is\nlike a code template parameterized through UI interactions.\n\nThis module builds on top of `Kino.JS.Live`, consequently keeping\nall of its component and communication mechanics. The additional\ncallbacks specify how the UI maps to source code.","ref":"Kino.SmartCell.html"},{"type":"behaviour","title":"Usage - Kino.SmartCell","doc":"Defining a custom cell is similar to writing a regular `Kino.JS.Live`\ncomponent, with a couple specifics.\n\nFirst, we only need to define callbacks, so there is no need for\nusing `Kino.JS.Live.new/2`. The `c:Kino.JS.Live.init/2` callback\nalways receives `t:attrs/0` as the first argument.\n\nSecond, we add a few new bits, namely `use Kino.SmartCell` and the\ntwo corresponding callback definitions.\n\nHere is an outline of a custom module\n\n    defmodule KinoDocs.CustomCell do\n      use Kino.JS\n      use Kino.JS.Live\n      use Kino.SmartCell, name: \"Our custom wizard\"\n\n      @impl true\n      def init(attrs, ctx) do\n        ...\n      end\n\n      # Other Kino.JS.Live callbacks\n      ...\n\n      @impl true\n      def to_attrs(ctx) do\n        ...\n      end\n\n      @impl true\n      def to_source(attrs) do\n        ...\n      end\n    end\n\nAdditionally, in order for Livebook to pick up the custom cell, we\nneed to register our module. This usually happens in `application.ex`\n\n    Kino.SmartCell.register(KinoDocs.CustomCell)","ref":"Kino.SmartCell.html#module-usage"},{"type":"behaviour","title":"Example - Kino.SmartCell","doc":"As a minimal example, that's how we can define a cell that allows\nediting the underlying code directly through a textarea.\n\n    defmodule KinoDocs.SmartCell.Plain do\n      use Kino.JS\n      use Kino.JS.Live\n      use Kino.SmartCell, name: \"Plain code editor\"\n\n      @impl true\n      def init(attrs, ctx) do\n        source = attrs[\"source\"] || \"\"\n        {:ok, assign(ctx, source: source)}\n      end\n\n      @impl true\n      def handle_connect(ctx) do\n        {:ok, %{source: ctx.assigns.source}, ctx}\n      end\n\n      @impl true\n      def handle_event(\"update\", %{\"source\" => source}, ctx) do\n        broadcast_event(ctx, \"update\", %{\"source\" => source})\n        {:noreply, assign(ctx, source: source)}\n      end\n\n      @impl true\n      def to_attrs(ctx) do\n        %{\"source\" => ctx.assigns.source}\n      end\n\n      @impl true\n      def to_source(attrs) do\n        attrs[\"source\"]\n      end\n\n      asset \"main.js\" do\n        \"\"\"\n        export function init(ctx, payload) {\n          ctx.importCSS(\"main.css\");\n\n          ctx.root.innerHTML = `\n              \n          `;\n\n          const textarea = ctx.root.querySelector(\"#source\");\n          textarea.value = payload.source;\n\n          textarea.addEventListener(\"change\", (event) => {\n            ctx.pushEvent(\"update\", { source: event.target.value });\n          });\n\n          ctx.handleEvent(\"update\", ({ source }) => {\n            textarea.value = source;\n          });\n\n          ctx.handleSync(() => {\n            // Synchronously invokes change listeners\n            document.activeElement &&\n              document.activeElement.dispatchEvent(new Event(\"change\"));\n          });\n        }\n        \"\"\"\n      end\n\n      asset \"main.css\" do\n        \"\"\"\n        #source {\n          box-sizing: border-box;\n          width: 100%;\n          min-height: 100px;\n        }\n        \"\"\"\n      end\n    end\n\nAnd then we would register it as\n\n    Kino.SmartCell.register(KinoDocs.SmartCell.Plain)\n\nNote that we register a synchronization handler on the client with\n`ctx.handleSync(() => ...)`. This optional handler is invoked before\nevaluation and it should flush any deferred UI changes to the server.\nIn our example we listen to textarea's \"change\" event, which is only\ntriggered on blur, so on synchronization we trigger it programmatically.","ref":"Kino.SmartCell.html#module-example"},{"type":"behaviour","title":"Collaborative editor - Kino.SmartCell","doc":"If a smart cell requires editing some code (like SQL), it may use\na dedicated editor instance managed by Livebook. The editor handles\nsyntax highlighting and collaborative editing, similarly to the\nbuilt-in cells.\n\nTo enable the editor, we need to include `:editor` configuration in\noptions returned from the `c:Kino.JS.Live.init/2` callback.\n\n    @impl true\n    def init(attrs, ctx) do\n      # ...\n      {:ok, ctx, editor: [source: \"\", language: \"elixir\"]}\n    end\n\nYou also need to define `c:handle_editor_change/2`, which usually\nsimply stores the new source in the context.","ref":"Kino.SmartCell.html#module-collaborative-editor"},{"type":"behaviour","title":"Example - Kino.SmartCell","doc":"Here is a minimal example, similar to the one before, but using the\neditor feature.\n\n    defmodule KinoDocs.SmartCell.Editor do\n      use Kino.JS\n      use Kino.JS.Live\n      use Kino.SmartCell, name: \"Built-in code editor\"\n\n      @impl true\n      def init(attrs, ctx) do\n        source = attrs[\"source\"] || \"\"\n        {:ok, assign(ctx, source: source), editor: [source: source]}\n      end\n\n      @impl true\n      def handle_connect(ctx) do\n        {:ok, %{}, ctx}\n      end\n\n      @impl true\n      def handle_editor_change(source, ctx) do\n        {:ok, assign(ctx, source: source)}\n      end\n\n      @impl true\n      def to_attrs(ctx) do\n        %{\"source\" => ctx.assigns.source}\n      end\n\n      @impl true\n      def to_source(attrs) do\n        attrs[\"source\"]\n      end\n\n      asset \"main.js\" do\n        \"\"\"\n        export function init(ctx, payload) {\n          ctx.importCSS(\"main.css\");\n\n          ctx.root.innerHTML = `Editor:`;\n        }\n        \"\"\"\n      end\n    end","ref":"Kino.SmartCell.html#module-example"},{"type":"behaviour","title":"Options - Kino.SmartCell","doc":"* `:source` - the initial editor source. Required\n\n  * `:language` - the editor language, used for syntax highlighting.\n    Defaults to `nil`\n\n  * `:placement` - editor placement within the smart cell, either\n    `:top` or `:bottom`. Defaults to `:bottom`\n\n  * `:intellisense_node` - a `{node, cookie}` atom tuple specifying\n    a remote node that should be introspected for editor intellisense.\n    This is only applicable when `:language` is Elixir. Defaults to\n    `nil`\n\nNote that you can programmatically reconfigure some of these options\nlater using `Kino.JS.Live.Context.reconfigure_smart_cell/2`.","ref":"Kino.SmartCell.html#module-options"},{"type":"behaviour","title":"Other options - Kino.SmartCell","doc":"Other than the editor configuration, the following options are\nsupported:\n\n  * `:reevaluate_on_change` - if the cell should be reevaluated\n    whenever the generated source code changes. This option may be\n    helpful in cases where the cell output is a crucial element of\n    the UI interactions. Defaults to `false`","ref":"Kino.SmartCell.html#module-other-options"},{"type":"function","title":"Kino.SmartCell.definitions/0","doc":"Returns a list of available smart cell definitions.","ref":"Kino.SmartCell.html#definitions/0"},{"type":"callback","title":"Kino.SmartCell.handle_editor_change/2","doc":"Invoked when the smart cell editor content changes.\n\nUsually you just want to put the new source in the corresponding\nassign.\n\nThis callback is required if the smart cell enables editor in the\n`c:Kino.JS.Live.init/2` configuration.","ref":"Kino.SmartCell.html#c:handle_editor_change/2"},{"type":"function","title":"Kino.SmartCell.prefixed_var_name/2","doc":"Generates unique variable names with the given prefix.\n\nWhen `var_name` is `nil`, allocates and returns the next available\nname. Otherwise, marks the given suffix as taken, provided that\n`var_name` has the given prefix.\n\nThis function can be used to generate default variable names during\nsmart cell initialization, so that don't overlap.","ref":"Kino.SmartCell.html#prefixed_var_name/2"},{"type":"function","title":"Kino.SmartCell.quoted_to_string/1","doc":"Converts the given AST to formatted code string.","ref":"Kino.SmartCell.html#quoted_to_string/1"},{"type":"function","title":"Kino.SmartCell.register/1","doc":"Registers a new smart cell.\n\nThis should usually be called in `application.ex` when starting\nthe application.","ref":"Kino.SmartCell.html#register/1"},{"type":"function","title":"Examples - Kino.SmartCell.register/1","doc":"Kino.SmartCell.register(KinoDocs.CustomCell)","ref":"Kino.SmartCell.html#register/1-examples"},{"type":"callback","title":"Kino.SmartCell.scan_binding/3","doc":"Invoked whenever the base evaluation context changes.\n\nThis callback receives the binding and environment available to the\nsmart cell code.\n\nNote that this callback runs asynchronously and it receives the PID\nof the smart cell server, so the result needs to be sent explicitly\nand handled using `c:Kino.JS.Live.handle_info/2`.\n\n**Important:** remember that data sent between processes is copied,\nso avoid sending large data structures. In particular, when looking\nat variables, instead of sending their values, extract and send\nonly the relevant metadata.\n\n**Important:** avoid any heavy work in this callback, as it runs in\nthe same process that evaluates code, so we don't want to block it.","ref":"Kino.SmartCell.html#c:scan_binding/3"},{"type":"callback","title":"Kino.SmartCell.scan_eval_result/2","doc":"Invoked when the smart cell code is evaluated.\n\nThis callback receives the result of an evaluation, either the\nreturn value or an exception if raised.\n\nThis callback runs asynchronously and has the same characteristics\nas `c:scan_binding/3`.","ref":"Kino.SmartCell.html#c:scan_eval_result/2"},{"type":"callback","title":"Kino.SmartCell.to_attrs/1","doc":"Invoked to compute the smart cell state as serializable attributes.","ref":"Kino.SmartCell.html#c:to_attrs/1"},{"type":"callback","title":"Kino.SmartCell.to_source/1","doc":"Invoked to generate source code based on the given attributes.\n\nA list of sources can be returned, in which case the sources are\njoined into a single source, however converting to a Code cell\nresults in multiple cells.","ref":"Kino.SmartCell.html#c:to_source/1"},{"type":"function","title":"Kino.SmartCell.valid_variable_name?/1","doc":"Checks if the given string is a valid Elixir variable name.","ref":"Kino.SmartCell.html#valid_variable_name?/1"},{"type":"type","title":"Kino.SmartCell.attrs/0","doc":"Attributes are an intermediate form of smart cell state, used to\npersist and restore cells.\n\nAttributes are computed using `c:to_attrs/1` and used to generate\nthe source code using `c:to_source/1`.\n\nNote that attributes are serialized and deserialized as JSON for\npersistence, hence make sure to use JSON-friendly data structures.\n\nPersisted attributes are passed to `c:Kino.JS.Live.init/2` as the\nfirst argument and should be used to restore the relevant state.","ref":"Kino.SmartCell.html#t:attrs/0"},{"type":"type","title":"Kino.SmartCell.eval_result/0","doc":"","ref":"Kino.SmartCell.html#t:eval_result/0"},{"type":"module","title":"Kino.Inspect","doc":"A struct wrapping any term for default rendering.\n\nThis is just a meta-struct that implements the `Kino.Render`\nprotocol, so that the wrapped value is rendered using the inspect\nprotocol.","ref":"Kino.Inspect.html"},{"type":"function","title":"Kino.Inspect.new/1","doc":"Wraps the given term.","ref":"Kino.Inspect.html#new/1"},{"type":"opaque","title":"Kino.Inspect.t/0","doc":"","ref":"Kino.Inspect.html#t:t/0"},{"type":"protocol","title":"Kino.Render","doc":"Protocol defining term formatting in the context of Livebook.","ref":"Kino.Render.html"},{"type":"function","title":"Kino.Render.to_livebook/1","doc":"Transforms the given value into a Livebook-compatible output.\n\nFor detailed description of the output format see `t:Livebook.Runtime.output/0`.\n\nWhen implementing the protocol for custom struct, you generally do\nnot need to worry about the output format. Instead, you can compose\nbuilt-in kinos and call `Kino.Render.to_livebook/1` to get the\nexpected representation.\n\nFor example, if we wanted to render a custom struct as a mermaid\ngraph, we could do this:\n\n    defimpl Kino.Render, for: Graph do\n      def to_livebook(graph) do\n        source = Graph.to_mermaid(graph)\n        mermaid_kino = Kino.Mermaid.new(source)\n        Kino.Render.to_livebook(mermaid_kino)\n      end\n    end\n\nIn many cases it is useful to show the default inspect representation\nalongside our custom visual representation. For this, we can use tabs:\n\n    defimpl Kino.Render, for: Graph do\n      def to_livebook(graph) do\n        source = Graph.to_mermaid(graph)\n        mermaid_kino = Kino.Mermaid.new(source)\n        inspect_kino = Kino.Inspect.new(image)\n        kino = Kino.Layout.tabs(Graph: mermaid_kino, Raw: inspect_kino)\n        Kino.Render.to_livebook(kino)\n      end\n    end","ref":"Kino.Render.html#to_livebook/1"},{"type":"type","title":"Kino.Render.t/0","doc":"All the types that implement this protocol.","ref":"Kino.Render.html#t:t/0"},{"type":"behaviour","title":"Kino.Table","doc":"A behaviour module for implementing tabular kinos.\n\nThis module implements table visualization and delegates data\nfetching and traversal to the behaviour implementation.","ref":"Kino.Table.html"},{"type":"callback","title":"Kino.Table.export_data/3","doc":"Exports the data for download.\n\nThe returned map must contain the binary, the file extension and the mime type.","ref":"Kino.Table.html#c:export_data/3"},{"type":"callback","title":"Kino.Table.get_data/2","doc":"Loads data matching the given specification.","ref":"Kino.Table.html#c:get_data/2"},{"type":"callback","title":"Kino.Table.init/1","doc":"Invoked once to initialize server state.","ref":"Kino.Table.html#c:init/1"},{"type":"function","title":"Kino.Table.new/3","doc":"Creates a new tabular kino using the given module as data\nspecification.","ref":"Kino.Table.html#new/3"},{"type":"function","title":"Options - Kino.Table.new/3","doc":"* `:export` - a function called to export the given kino to Markdown.\n    This works the same as `Kino.JS.new/3`, except the function\n    receives the state as an argument","ref":"Kino.Table.html#new/3-options"},{"type":"callback","title":"Kino.Table.on_update/2","doc":"Invoked to update state with new data.\n\nThis callback is called in response to `update/2`.","ref":"Kino.Table.html#c:on_update/2"},{"type":"function","title":"Kino.Table.update/2","doc":"Updates the table with new data.\n\nAn arbitrary update event can be used and it is then handled by\nthe `c:on_update/2` callback.","ref":"Kino.Table.html#update/2"},{"type":"type","title":"Kino.Table.column/0","doc":"","ref":"Kino.Table.html#t:column/0"},{"type":"type","title":"Kino.Table.info/0","doc":"","ref":"Kino.Table.html#t:info/0"},{"type":"type","title":"Kino.Table.rows_spec/0","doc":"","ref":"Kino.Table.html#t:rows_spec/0"},{"type":"type","title":"Kino.Table.state/0","doc":"","ref":"Kino.Table.html#t:state/0"},{"type":"type","title":"Kino.Table.t/0","doc":"","ref":"Kino.Table.html#t:t/0"},{"type":"module","title":"Kino.Test","doc":"Conveniences for testing custom Kino components.\n\nIn practice, `Kino.JS.Live` kinos communicate with Livebook via\nthe group leader. During tests, Livebook is out of the equation,\nso we need to mimic this side of the communication. To do so, add\nthe following setup to your test module:\n\n    import Kino.Test\n\n    setup :configure_livebook_bridge","ref":"Kino.Test.html"},{"type":"macro","title":"Kino.Test.assert_broadcast_event/4","doc":"Asserts a `Kino.JS.Live` kino will broadcast an event within\n`timeout`.","ref":"Kino.Test.html#assert_broadcast_event/4"},{"type":"macro","title":"Examples - Kino.Test.assert_broadcast_event/4","doc":"assert_broadcast_event(kino, \"bump\", %{by: 2})","ref":"Kino.Test.html#assert_broadcast_event/4-examples"},{"type":"macro","title":"Kino.Test.assert_output/2","doc":"Asserts the given output is sent within `timeout`.","ref":"Kino.Test.html#assert_output/2"},{"type":"macro","title":"Examples - Kino.Test.assert_output/2","doc":"assert_output({:markdown, \"_hey_\"})","ref":"Kino.Test.html#assert_output/2-examples"},{"type":"macro","title":"Kino.Test.assert_output_to/3","doc":"Asserts the given output is sent directly to the given client within\n`timeout`.","ref":"Kino.Test.html#assert_output_to/3"},{"type":"macro","title":"Examples - Kino.Test.assert_output_to/3","doc":"assert_output_to(\"client1\", {:markdown, \"_hey_\"})","ref":"Kino.Test.html#assert_output_to/3-examples"},{"type":"macro","title":"Kino.Test.assert_output_to_clients/2","doc":"Asserts the given output is sent directly to all clients within `timeout`.","ref":"Kino.Test.html#assert_output_to_clients/2"},{"type":"macro","title":"Examples - Kino.Test.assert_output_to_clients/2","doc":"assert_output_to(\"client1\", {:markdown, \"_hey_\"})","ref":"Kino.Test.html#assert_output_to_clients/2-examples"},{"type":"macro","title":"Kino.Test.assert_send_event/4","doc":"Asserts a `Kino.JS.Live` kino will send an event within `timeout`\nto the caller.","ref":"Kino.Test.html#assert_send_event/4"},{"type":"macro","title":"Examples - Kino.Test.assert_send_event/4","doc":"assert_send_event(kino, \"pong\", %{})","ref":"Kino.Test.html#assert_send_event/4-examples"},{"type":"macro","title":"Kino.Test.assert_smart_cell_update/4","doc":"Asserts a smart cell update will be broadcasted within `timeout`.\n\nMatches against the source and attribute that are reported as part\nof the update.\n\nIf the `source` argument is a string, that string is compared in an\nexact match against the Kino's source.\n\nAlternatively, the `source` argument can be used to bind a variable\nto the Kino's source, allowing for custom assertions against the\nsource.","ref":"Kino.Test.html#assert_smart_cell_update/4"},{"type":"macro","title":"Examples - Kino.Test.assert_smart_cell_update/4","doc":"assert_smart_cell_update(kino, %{\"variable\" => \"x\", \"number\" => 10}, \"x = 10\")\n\n    assert_smart_cell_update(kino, %{\"variable\" => \"x\", \"number\" => 10}, source)\n    assert source =~ \"10\"","ref":"Kino.Test.html#assert_smart_cell_update/4-examples"},{"type":"function","title":"Kino.Test.configure_livebook_bridge/1","doc":"","ref":"Kino.Test.html#configure_livebook_bridge/1"},{"type":"function","title":"Kino.Test.connect/3","doc":"Connects to a `Kino.JS.Live` kino and returns the initial data.\n\nIf `resolve_fun` is given, it runs after sending the connection\nrequest and before awaiting for the reply.","ref":"Kino.Test.html#connect/3"},{"type":"function","title":"Examples - Kino.Test.connect/3","doc":"data = connect(kino)\n    assert data == %{count: 1}","ref":"Kino.Test.html#connect/3-examples"},{"type":"function","title":"Kino.Test.push_event/3","doc":"Sends a client event to a `Kino.JS.Live` kino.","ref":"Kino.Test.html#push_event/3"},{"type":"function","title":"Examples - Kino.Test.push_event/3","doc":"push_event(kino, \"bump\", %{\"by\" => 2})","ref":"Kino.Test.html#push_event/3-examples"},{"type":"function","title":"Kino.Test.start_smart_cell!/2","doc":"Starts a smart cell defined by the given module.\n\nReturns a `Kino.JS.Live` kino for interacting with the cell, as\nwell as the initial source.","ref":"Kino.Test.html#start_smart_cell!/2"},{"type":"function","title":"Examples - Kino.Test.start_smart_cell!/2","doc":"{kino, source} = start_smart_cell!(Kino.SmartCell.Custom, %{\"key\" => \"value\"})","ref":"Kino.Test.html#start_smart_cell!/2-examples"},{"type":"exception","title":"Kino.FS.ForbiddenError","doc":"Exception raised when access to a notebook file is forbidden.","ref":"Kino.FS.ForbiddenError.html"},{"type":"exception","title":"Kino.InterruptError","doc":"Exception raised to stop evaluation in expected manner.","ref":"Kino.InterruptError.html"},{"type":"type","title":"Kino.InterruptError.t/0","doc":"","ref":"Kino.InterruptError.html#t:t/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}